Package: llmcoder
Title: LLM-Powered Code Generation, Error Fixing, and Chat for
        'RStudio'
Version: 1.2.0
Maintainer: Shiyang Zheng <shiyang.zheng@nottingham.ac.uk>
Author: Shiyang Zheng [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-0511-4683>)
Authors@R: 
    person(given  = "Shiyang",
           family = "Zheng",
           email  = "shiyang.zheng@nottingham.ac.uk",
           role   = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-0511-4683"))
Description: An 'RStudio' addin that integrates large language model (LLM)
    assistance directly into the code-editing workflow. Features include:
    (1) generate R code from inline comments; (2) obtain LLM-assisted fixes for
    console errors; (3) insert plain-English explanations of selected code
    blocks; (4) a multi-turn Chat Panel with session-context awareness
    (loaded packages, global objects, source editor contents, console history).
    Supports 'OpenAI', 'Anthropic' (Claude), 'DeepSeek', 'Groq',
    'Together AI', 'OpenRouter', 'Ollama' (fully local, no API key required),
    and any 'OpenAI'-compatible custom endpoint (e.g. 'LM Studio', 'vLLM',
    'llama.cpp').
License: MIT + file LICENSE
Depends: R (>= 4.1.0)
Encoding: UTF-8
RoxygenNote: 7.3.2
URL: https://github.com/ShiyangZheng/llmcoder
BugReports: https://github.com/ShiyangZheng/llmcoder/issues
Imports: rstudioapi (>= 0.13), httr2 (>= 1.0.0), miniUI (>= 0.1.1),
        shiny (>= 1.7.0), stringi (>= 1.7.0), stringr (>= 1.5.0), rlang
        (>= 1.0.0), htmltools (>= 0.5.0), jsonlite (>= 1.8.0)
Suggests: testthat (>= 3.0.0), withr (>= 2.5.0)
NeedsCompilation: no
Packaged: 2026-05-02 20:42:57 UTC; admin
Repository: CRAN
Date/Publication: 2026-05-05 14:06:13 UTC
Built: R 4.5.2; ; 2026-05-05 16:43:34 UTC; unix

Package: llmcoder
Title: LLM-Powered Code Generation, Error Fixing, and Chat for
        'RStudio'
Version: 1.2.0
Maintainer: Shiyang Zheng <shiyang.zheng@nottingham.ac.uk>
Author: Shiyang Zheng [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-0511-4683>)
Authors@R: 
    person(given  = "Shiyang",
           family = "Zheng",
           email  = "shiyang.zheng@nottingham.ac.uk",
           role   = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-0511-4683"))
Description: An 'RStudio' addin that integrates large language model (LLM)
    assistance directly into the code-editing workflow. Features include:
    (1) generate R code from inline comments; (2) obtain LLM-assisted fixes for
    console errors; (3) insert plain-English explanations of selected code
    blocks; (4) a multi-turn Chat Panel with session-context awareness
    (loaded packages, global objects, source editor contents, console history).
    Supports 'OpenAI', 'Anthropic' (Claude), 'DeepSeek', 'Groq',
    'Together AI', 'OpenRouter', 'Ollama' (fully local, no API key required),
    and any 'OpenAI'-compatible custom endpoint (e.g. 'LM Studio', 'vLLM',
    'llama.cpp').
License: MIT + file LICENSE
Depends: R (>= 4.1.0)
Encoding: UTF-8
RoxygenNote: 7.3.2
URL: https://github.com/ShiyangZheng/llmcoder
BugReports: https://github.com/ShiyangZheng/llmcoder/issues
Imports: rstudioapi (>= 0.13), httr2 (>= 1.0.0), miniUI (>= 0.1.1),
        shiny (>= 1.7.0), stringi (>= 1.7.0), stringr (>= 1.5.0), rlang
        (>= 1.0.0), htmltools (>= 0.5.0), jsonlite (>= 1.8.0)
Suggests: testthat (>= 3.0.0), withr (>= 2.5.0)
NeedsCompilation: no
Packaged: 2026-05-02 20:42:57 UTC; admin
Repository: CRAN
Date/Publication: 2026-05-05 14:06:13 UTC
Built: R 4.6.0; ; 2026-05-05 16:40:49 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(print,llmcoder_config)
export(addin_chat_panel)
export(addin_explain_code)
export(addin_fix_console_error)
export(addin_fix_selected_error)
export(addin_generate_from_comment)
export(addin_generate_with_preview)
export(addin_settings)
export(llmcoder_config)
export(llmcoder_setup)
export(ollama_list_models)
export(session_context_prompt)
export(session_context_report)
importFrom(htmltools,HTML)
importFrom(jsonlite,fromJSON)
importFrom(shiny,dialogViewer)
importFrom(shiny,req)
import(stringi)
importFrom(utils,getFromNamespace)
importFrom(utils,URLencode)

# ============================================================
#  llmcoder — Shiny mini-gadgets
# ============================================================

# ---- 1. Code preview gadget (generate) -------------------------------

#' @keywords internal
preview_gadget <- function(comment, code, row, indent, ctx_id) {

  ui <- miniUI::miniPage(
    shiny::tags$head(shiny::tags$style(gadget_css())),
    miniUI::gadgetTitleBar(
      title = shiny::tags$span(
        shiny::tags$span("\U1F916", style = "margin-right:6px;"),
        shiny::tags$em(comment, style = "font-style:italic; color:#555;")
      ),
      right = miniUI::miniTitleBarButton("insert", "Insert \u2193", primary = TRUE)
    ),
    miniUI::miniContentPanel(
      shiny::tags$textarea(
        id    = "code_edit",
        rows  = 20,
        class = "code-area",
        code
      ),
      shiny::tags$p(
        "\u2139\ufe0f Edit freely, then click \u2018Insert\u2019.",
        class = "hint"
      )
    )
  )

  server <- function(input, output, session) {
    shiny::observeEvent(input$insert, {
      if (!is.null(input$code_edit) && nchar(trimws(input$code_edit)) > 0)
        insert_after_row(input$code_edit, row, indent)
      notify("Code inserted.")
      shiny::stopApp(invisible(NULL))
    })
    shiny::observeEvent(input$cancel, shiny::stopApp(invisible(NULL)))
  }

  shiny::runGadget(ui, server,
    viewer = shiny::dialogViewer("LLMcoder Preview", width = 720, height = 540))
}


# ---- 2. Fix-error preview gadget ------------------------------------

#' @keywords internal
fix_preview_gadget <- function(error_msg, original, fixed, ctx) {

  ui <- miniUI::miniPage(
    shiny::tags$head(shiny::tags$style(gadget_css())),
    miniUI::gadgetTitleBar(
      title = "\U1F527  LLMcoder \u2014 Error Fix Preview",
      right = miniUI::miniTitleBarButton("apply", "Apply Fix", primary = TRUE)
    ),
    miniUI::miniContentPanel(
      shiny::tags$div(
        class = "error-box",
        shiny::tags$strong("Error:"),
        shiny::tags$pre(
          substr(error_msg, 1, 400),
          style = "margin:4px 0 0; font-size:11px; white-space:pre-wrap;"
        )
      ),
      shiny::tags$p(
        shiny::tags$strong("Fixed code"),
        " (lines marked ", shiny::tags$code("# FIX:"),
        " show what changed) \u2014 edit before applying:",
        class = "hint"
      ),
      shiny::tags$textarea(
        id    = "fixed_edit",
        rows  = 22,
        class = "code-area",
        fixed
      )
    )
  )

  server <- function(input, output, session) {
    shiny::observeEvent(input$apply, {
      edited <- input$fixed_edit
      if (!is.null(edited) && nchar(trimws(edited)) > 0) {
        # Replace entire document: use row count from ctx (0-indexed end)
        n_lines <- length(ctx$contents)
        # document_position rows are 1-based; last position is (n_lines, 0)
        # means "beginning of the line AFTER the last line", which safely
        # selects the full content even if the last line has no trailing newline.
        rstudioapi::modifyRange(
          location = rstudioapi::document_range(
            rstudioapi::document_position(1L, 1L),
            rstudioapi::document_position(n_lines, .Machine$integer.max)
          ),
          text = paste0(edited, "\n"),
          id   = ctx$id
        )
        notify("Fix applied.")
      }
      shiny::stopApp(invisible(NULL))
    })
    shiny::observeEvent(input$cancel, shiny::stopApp(invisible(NULL)))
  }

  shiny::runGadget(ui, server,
    viewer = shiny::dialogViewer("LLMcoder Fix Preview", width = 760, height = 640))
}


# ---- 3. Settings gadget -----------------------------------------------

#' @keywords internal
settings_gadget <- function() {

  # Current values
  cur_provider    <- getOption("llmcoder.provider",      "openai")
  cur_model       <- getOption("llmcoder.model",         "")
  cur_key         <- getOption("llmcoder.api_key",       Sys.getenv("LLMCODER_API_KEY"))
  cur_ollama_url  <- getOption("llmcoder.ollama_url",    "http://localhost:11434")
  cur_custom_url  <- getOption("llmcoder.custom_url",    "")
  cur_ctx_lines   <- getOption("llmcoder.context_lines", 40L)

  model_defaults <- list(
    openai     = "gpt-4o-mini",
    anthropic  = "claude-sonnet-4-20250514",
    deepseek   = "deepseek-chat",
    groq       = "llama-3.3-70b-versatile",
    together   = "meta-llama/Llama-3-70b-chat-hf",
    openrouter = "openai/gpt-4o-mini",
    ollama     = "llama3",
    custom     = ""
  )

  provider_choices <- c(
    "OpenAI"                          = "openai",
    "Anthropic (Claude)"              = "anthropic",
    "DeepSeek"                        = "deepseek",
    "Groq (fast open models)"         = "groq",
    "Together AI (open models)"       = "together",
    "OpenRouter (100+ models)"        = "openrouter",
    "Ollama (local)"                  = "ollama",
    "Custom / LM Studio / vLLM"       = "custom"
  )

  ui <- miniUI::miniPage(
    shiny::tags$head(shiny::tags$style(gadget_css())),
    miniUI::gadgetTitleBar(
      "\U2699\uFE0F  LLMcoder Settings",
      right = miniUI::miniTitleBarButton("save", "Save", primary = TRUE)
    ),
    miniUI::miniContentPanel(
      padding = 15,

      # --- Provider --------------------------------------------------
      shiny::selectInput("provider", "Provider",
        choices  = provider_choices,
        selected = cur_provider
      ),

      # --- Model (text + refresh for Ollama) -------------------------
      shiny::uiOutput("model_ui"),

      # --- API key (hidden for Ollama) -------------------------------
      shiny::uiOutput("key_ui"),

      # --- Ollama URL (shown only when provider == ollama) -----------
      shiny::uiOutput("ollama_url_ui"),

      # --- Custom base URL (shown only when provider == custom) ------
      shiny::uiOutput("custom_url_ui"),

      shiny::hr(),

      # --- Context lines slider -------------------------------------
      shiny::sliderInput(
        "ctx_lines",
        "Context lines sent to LLM (lines of code above cursor)",
        min   = 0, max = 150, value = cur_ctx_lines, step = 5
      ),
      shiny::tags$p(
        "More context = better suggestions, but slower and uses more tokens.",
        class = "hint"
      ),

      shiny::hr(),

      shiny::checkboxInput("save_profile",
        "Persist settings to ~/.Rprofile (survives restarts)", value = TRUE)
    )
  )

  server <- function(input, output, session) {

    # ---- Dynamic model UI ----
    output$model_ui <- shiny::renderUI({
      prov  <- input$provider %||% cur_provider
      def_m <- model_defaults[[prov]] %||% ""
      cur_m <- if (nchar(cur_model) > 0) cur_model else def_m

      if (prov == "ollama") {
        shiny::tagList(
          shiny::fluidRow(
            shiny::column(9,
              shiny::textInput("model", "Model (Ollama)",
                value       = cur_m,
                placeholder = "llama3, mistral, qwen2.5-coder, ...")
            ),
            shiny::column(3,
              shiny::tags$br(),
              shiny::actionButton("refresh_ollama", "\U1F504 Refresh",
                style = "margin-top:6px; width:100%;")
            )
          ),
          shiny::uiOutput("ollama_model_picker")
        )
      } else {
        shiny::textInput("model", "Model",
          value       = cur_m,
          placeholder = def_m)
      }
    })

    # ---- Ollama model picker (populated after Refresh) ----
    ollama_models <- shiny::reactiveVal(character(0))

    shiny::observeEvent(input$refresh_ollama, {
      url <- trimws(input$ollama_url %||% cur_ollama_url)
      if (nchar(url) == 0) url <- cur_ollama_url
      old <- getOption("llmcoder.ollama_url")
      options(llmcoder.ollama_url = url)
      models <- ollama_list_models()
      options(llmcoder.ollama_url = old)
      if (!is.null(models) && length(models) > 0) {
        ollama_models(models)
        shiny::showNotification(
          paste(length(models), "model(s) found."),
          type = "message", duration = 3
        )
      } else {
        shiny::showNotification(
          "No models found. Is `ollama serve` running?",
          type = "warning", duration = 5
        )
      }
    })

    output$ollama_model_picker <- shiny::renderUI({
      models <- ollama_models()
      if (length(models) == 0) return(NULL)
      shiny::tagList(
        shiny::selectInput("ollama_model_select",
          "Available models (click to populate Model field):",
          choices = models, selected = models[1]),
        shiny::tags$p(
          "Select a model above to populate the Model field automatically.",
          class = "hint"
        )
      )
    })

    # When user picks from the list, update the text field
    shiny::observeEvent(input$ollama_model_select, {
      shiny::updateTextInput(session, "model", value = input$ollama_model_select)
    })

    # ---- API key UI ----
    output$key_ui <- shiny::renderUI({
      prov <- input$provider %||% cur_provider
      if (prov == "ollama") return(NULL)
      placeholder <- switch(prov,
        openai     = "sk-...",
        anthropic  = "sk-ant-...",
        deepseek   = "sk-...",
        groq       = "gsk_...",
        together   = "your-together-api-key",
        openrouter = "sk-or-...",
        custom     = "your-key-or-empty",
        "your-api-key"
      )
      link_txt <- switch(prov,
        openai     = "Get key at platform.openai.com",
        anthropic  = "Get key at console.anthropic.com",
        deepseek   = "Get key at platform.deepseek.com",
        groq       = "Get key at console.groq.com",
        together   = "Get key at api.together.xyz",
        openrouter = "Get key at openrouter.ai/keys",
        NULL
      )
      shiny::tagList(
        shiny::passwordInput("api_key", "API Key",
          value       = cur_key,
          placeholder = placeholder),
        if (!is.null(link_txt))
          shiny::tags$p(link_txt, class = "hint")
      )
    })

    # ---- Ollama URL UI ----
    output$ollama_url_ui <- shiny::renderUI({
      if ((input$provider %||% cur_provider) != "ollama") return(NULL)
      shiny::tagList(
        shiny::textInput("ollama_url", "Ollama base URL",
          value       = cur_ollama_url,
          placeholder = "http://localhost:11434"),
        shiny::tags$p(
          "Default: http://localhost:11434  \u2014  start with: ollama serve",
          class = "hint"
        )
      )
    })

    # ---- Custom URL UI ----
    output$custom_url_ui <- shiny::renderUI({
      if ((input$provider %||% cur_provider) != "custom") return(NULL)
      shiny::tagList(
        shiny::textInput("custom_url", "Custom OpenAI-compatible base URL",
          value       = cur_custom_url,
          placeholder = "http://localhost:1234/v1"),
        shiny::tags$p(
          "Compatible with LM Studio, vLLM, llama.cpp server, Mistral API, etc.",
          class = "hint"
        )
      )
    })

    # ---- Save ----
    shiny::observeEvent(input$save, {
      prov       <- input$provider   %||% cur_provider
      model_val  <- input$model      %||% (model_defaults[[prov]] %||% "")
      key_val    <- input$api_key    %||% cur_key
      ctx_val    <- as.integer(input$ctx_lines %||% cur_ctx_lines)
      ollama_val <- input$ollama_url %||% cur_ollama_url
      custom_val <- input$custom_url %||% cur_custom_url

      options(
        llmcoder.provider      = prov,
        llmcoder.model         = model_val,
        llmcoder.api_key       = key_val,
        llmcoder.context_lines = ctx_val,
        llmcoder.ollama_url    = ollama_val,
        llmcoder.custom_url    = custom_val
      )

      if (isTRUE(input$save_profile)) {
        write_rprofile(prov, model_val, key_val, ctx_val, ollama_val, custom_val)
      } else {
        notify("Settings applied for this session only (not saved to .Rprofile).")
      }

      shiny::stopApp(invisible(NULL))
    })

    shiny::observeEvent(input$cancel, shiny::stopApp(invisible(NULL)))
  }

  shiny::runGadget(ui, server,
    viewer = shiny::dialogViewer("LLMcoder Settings", width = 500, height = 680))
}


# ---- Helpers -----------------------------------------------------------------

#' Write llmcoder options to ~/.Rprofile
#'
#' Writes (or replaces) an `# --- llmcoder ---` block in the user's
#' `~/.Rprofile` so that llmcoder settings persist across R sessions.
#'
#' @param provider   Character.  Provider identifier (see [llmcoder_setup()]).
#' @param model      Character.  Model name.
#' @param api_key    Character.  API key (may be `""` for Ollama).
#' @param ctx_lines  Integer.    Number of context lines.
#' @param ollama_url Character.  Ollama base URL.
#' @param custom_url Character.  Custom endpoint base URL.
#' @return Invisible `NULL`.  Called for its side-effect of writing to
#'   `~/.Rprofile`.
#' @examples
#' \dontrun{
#' write_rprofile(
#'   provider   = "ollama",
#'   model      = "llama3",
#'   api_key    = "",
#'   ctx_lines  = 40L,
#'   ollama_url = "http://localhost:11434",
#'   custom_url = ""
#' )
#' }
#' @keywords internal
write_rprofile <- function(provider, model, api_key, ctx_lines,
                            ollama_url, custom_url) {
  profile_path <- path.expand("~/.Rprofile")

  new_block <- c(
    "",
    "# --- llmcoder settings (written by addin_settings) ---",
    paste0('options(llmcoder.provider      = "', provider,   '")'),
    paste0('options(llmcoder.model         = "', model,      '")'),
    paste0('options(llmcoder.api_key       = "', api_key,    '")'),
    paste0('options(llmcoder.context_lines = ',  ctx_lines,  'L)'),
    paste0('options(llmcoder.ollama_url    = "', ollama_url, '")'),
    paste0('options(llmcoder.custom_url    = "', custom_url, '")')
  )

  existing <- if (file.exists(profile_path))
    readLines(profile_path, warn = FALSE)
  else
    character(0)

  # Remove old llmcoder block
  keep <- !stringr::str_detect(
    existing,
    "llmcoder\\.(provider|model|api_key|context_lines|ollama_url|custom_url)|llmcoder settings"
  )
  writeLines(c(existing[keep], new_block), profile_path)
  notify("Settings saved to ~/.Rprofile")
}

#' Shared CSS for all gadgets
#' @keywords internal
gadget_css <- function() {
  "
  body { font-family: -apple-system, BlinkMacSystemFont, 'Segoe UI', sans-serif; font-size: 13px; }
  .code-area {
    width: 100%; font-family: 'Fira Code', 'JetBrains Mono', 'Courier New', monospace;
    font-size: 12.5px; border: 1px solid #ddd; border-radius: 5px;
    padding: 10px; box-sizing: border-box; resize: vertical;
    background: #f8f8f8; line-height: 1.5; color: #333;
  }
  .code-area:focus { outline: none; border-color: #4a9eff; box-shadow: 0 0 0 2px rgba(74,158,255,.2); }
  .hint { color: #888; font-size: 11px; margin-top: 5px; }
  .error-box {
    background: #fff5f5; border: 1px solid #f5c6cb; border-radius: 5px;
    padding: 8px 12px; margin-bottom: 10px;
  }
  "
}

# Null-coalescing helper
`%||%` <- function(a, b) if (!is.null(a) && length(a) > 0 && nchar(as.character(a)[1]) > 0) a else b

# ============================================================
#  llmcoder — RStudio editor utilities
# ============================================================

#' Get the active source editor context
#'
#' Wraps [rstudioapi::getSourceEditorContext()] with a check that RStudio is
#' available.  Called by all addin entry points before any other operation.
#'
#' @return An `rstudio_editor_context` object (a list returned by
#'   [rstudioapi::getSourceEditorContext()]).
#' @keywords internal
get_editor_ctx <- function() {
  if (!rstudioapi::isAvailable()) {
    stop("This addin requires RStudio.", call. = FALSE)
  }
  rstudioapi::getSourceEditorContext()
}


#' Extract the comment text at the current cursor position
#'
#' Reads the line at the cursor position from the active editor context and
#' returns its components.  Throws an informative error if the cursor is not
#' positioned on a comment line (i.e., a line starting with `#`, possibly
#' preceded by whitespace).
#'
#' @param ctx An `rstudio_editor_context` object from
#'   [rstudioapi::getSourceEditorContext()].
#'
#' @return A named list with four components:
#' \describe{
#'   \item{`comment`}{Character.  The comment text with the leading `#`
#'     character(s) and optional space stripped.}
#'   \item{`row`}{Integer.  1-based row index of the comment line in the
#'     document.}
#'   \item{`full_line`}{Character.  The raw full-line text as it appears in
#'     the editor.}
#'   \item{`indent`}{Character.  The leading whitespace of the line
#'     (used to preserve indentation when inserting generated code).}
#' }
#'
#' @keywords internal
extract_comment_at_cursor <- function(ctx) {
  row      <- ctx$selection[[1]]$range$start[["row"]]
  lines    <- ctx$contents
  line_txt <- lines[[row]]

  # Must be a comment line (possibly with leading whitespace)
  if (!stringr::str_detect(line_txt, "^\\s*#")) {
    stop(
      "Cursor is not on a comment line.\n",
      "Place your cursor on a line starting with # and try again.",
      call. = FALSE
    )
  }

  # Strip leading whitespace + the # character(s) and optional space
  comment_text <- stringr::str_replace(line_txt, "^\\s*#+\\s?", "")

  list(
    comment   = comment_text,
    row       = row,
    full_line = line_txt,
    indent    = stringr::str_extract(line_txt, "^\\s*")
  )
}


#' Collect N lines of surrounding code above the cursor
#'
#' Returns a character string containing the `n` lines of source code
#' immediately above the comment line, joined by newlines.  This is sent to the
#' LLM as context so that it can infer variable names, existing code style, and
#' already-loaded packages.
#'
#' @param ctx  An `rstudio_editor_context` object.
#' @param row  Integer.  1-based row of the comment line (context is taken from
#'   rows `max(1, row - n)` to `row - 1`).
#' @param n    Integer.  Maximum number of context lines (default `30`).
#'
#' @return A single character string (may be `""` if `row == 1`).
#' @keywords internal
gather_context <- function(ctx, row, n = 30) {
  lines     <- ctx$contents
  start_row <- max(1L, row - n)
  end_row   <- max(1L, row - 1L)

  if (start_row > end_row) return("")

  paste(lines[start_row:end_row], collapse = "\n")
}


#' Insert text immediately after a given row in the editor
#'
#' Inserts one or more lines of text at the beginning of the row that follows
#' `row` in the currently active source editor.  Each line is prepended with
#' `indent` to match the indentation level of the originating comment.
#'
#' @param text   Character.  Code to insert; may contain newlines.
#' @param row    Integer.  1-based row *after* which the text is inserted.
#' @param indent Character.  Leading whitespace prepended to every inserted
#'   line (default `""`).
#'
#' @return Invisible `NULL` (called for side-effects).
#' @keywords internal
insert_after_row <- function(text, row, indent = "") {
  # Normalise: strip trailing blank lines
  text <- stringr::str_trim(text, side = "right")

  # Indent each line to match the comment
  indented <- paste(
    paste0(indent, stringr::str_split(text, "\n")[[1]]),
    collapse = "\n"
  )

  # Insert at the beginning of the next line
  insert_pos <- rstudioapi::document_position(row + 1L, 1L)

  rstudioapi::insertText(
    location = insert_pos,
    text     = paste0(indented, "\n"),
    id       = rstudioapi::getSourceEditorContext()$id
  )
}


#' Emit a status message to the R console
#'
#' Prefixes the message with `[llmcoder] ` so users can distinguish addin
#' output from their own code output.
#'
#' @param msg Character.  The message text.
#' @return Invisible `NULL`.
#' @keywords internal
notify <- function(msg) {
  message("[llmcoder] ", msg)
}

# ============================================================
#  llmcoder — session configuration helpers
# ============================================================

#' Configure LLMcoder for the current session
#'
#' Sets the LLM provider, API key, model, and related options for the current R
#' session.  For **permanent** configuration that survives restarts, use
#' **Addins > LLMcoder Settings**, which writes to \code{~/.Rprofile}.
#'
#' @param provider Character.  One of `"openai"`, `"anthropic"`,
#'   `"deepseek"`, `"ollama"`, `"groq"`, `"together"`, `"openrouter"`, or
#'   `"custom"`.
#' @param api_key Character.  Your API key.  Not required when
#'   `provider = "ollama"`.
#' @param model Character.  Model identifier.  If `NULL` or `""`, a sensible
#'   default is chosen for the provider (see **Details**).
#' @param context_lines Integer.  Number of lines of code above the cursor that
#'   are sent as context to the LLM (default `40`).  Higher values improve
#'   suggestion quality but increase latency and token cost.
#' @param ollama_url Character.  Base URL of the Ollama server (default
#'   `"http://localhost:11434"`).  Only used when `provider = "ollama"`.
#' @param custom_url Character.  Base URL of a custom OpenAI-compatible server
#'   (e.g. `"http://localhost:1234/v1"` for LM Studio).  Only used when
#'   `provider = "custom"`.
#'
#' @return Invisible `NULL`.
#'
#' @details
#' **Provider defaults:**
#'
#' | Provider      | Default model                        | Notes                         |
#' |---------------|--------------------------------------|-------------------------------|
#' | `openai`      | `gpt-4o-mini`                        | Fast, cost-effective          |
#' | `anthropic`   | `claude-sonnet-4-20250514`           | Strongest reasoning           |
#' | `deepseek`    | `deepseek-chat`                      | Very cheap, great code quality|
#' | `ollama`      | `llama3`                             | No API key, fully local       |
#' | `groq`        | `llama-3.3-70b-versatile`            | Extremely fast inference      |
#' | `together`    | `meta-llama/Llama-3-70b-chat-hf`    | Large open-source model choice|
#' | `openrouter`  | `openai/gpt-4o-mini`                 | Unified gateway for 100+ models|
#' | `custom`      | `""` (must specify)                  | Any OpenAI-compat endpoint    |
#'
#' @examples
#' \dontrun{
#' # OpenAI
#' llmcoder_setup("openai", api_key = Sys.getenv("OPENAI_API_KEY"))
#' llmcoder_setup("openai", api_key = Sys.getenv("OPENAI_API_KEY"), model = "gpt-4o")
#'
#' # Anthropic Claude
#' llmcoder_setup("anthropic", api_key = Sys.getenv("ANTHROPIC_API_KEY"))
#'
#' # DeepSeek (cheapest, excellent code quality)
#' llmcoder_setup("deepseek", api_key = Sys.getenv("DEEPSEEK_API_KEY"))
#'
#' # Ollama — fully local, no API key needed
#' llmcoder_setup("ollama", model = "qwen2.5-coder:7b")
#' llmcoder_setup("ollama", model = "codellama:13b",
#'                ollama_url = "http://192.168.1.10:11434")  # remote server
#'
#' # Groq — extremely fast inference on open models
#' llmcoder_setup("groq",
#'   api_key = Sys.getenv("GROQ_API_KEY"),
#'   model   = "llama-3.3-70b-versatile")
#'
#' # Together AI — wide open-source model selection
#' llmcoder_setup("together",
#'   api_key = Sys.getenv("TOGETHER_API_KEY"),
#'   model   = "mistralai/Mixtral-8x7B-Instruct-v0.1")
#'
#' # OpenRouter — unified gateway, supports 100+ models
#' llmcoder_setup("openrouter",
#'   api_key = Sys.getenv("OPENROUTER_API_KEY"),
#'   model   = "anthropic/claude-3.5-sonnet")
#'
#' # LM Studio or any OpenAI-compatible local server
#' llmcoder_setup("custom",
#'   api_key    = "lm-studio",
#'   model      = "local-model",
#'   custom_url = "http://localhost:1234/v1")
#'
#' # Reduce context window to save tokens
#' llmcoder_setup("openai",
#'   api_key       = Sys.getenv("OPENAI_API_KEY"),
#'   context_lines = 20L)
#' }
#'
#' @seealso [llmcoder_config()], [addin_settings()]
#' @export
llmcoder_setup <- function(
    provider      = c("openai", "anthropic", "deepseek",
                      "ollama", "groq", "together", "openrouter", "custom"),
    api_key       = NULL,
    model         = NULL,
    context_lines = 40L,
    ollama_url    = "http://localhost:11434",
    custom_url    = "") {

  provider <- match.arg(provider)

  local_providers <- c("ollama")
  if (!provider %in% local_providers &&
      (is.null(api_key) || nchar(api_key) == 0)) {
    stop("Please supply an api_key (not needed for 'ollama').", call. = FALSE)
  }

  defaults <- c(
    openai     = "gpt-4o-mini",
    anthropic  = "claude-sonnet-4-20250514",
    deepseek   = "deepseek-chat",
    ollama     = "llama3",
    groq       = "llama-3.3-70b-versatile",
    together   = "meta-llama/Llama-3-70b-chat-hf",
    openrouter = "openai/gpt-4o-mini",
    custom     = ""
  )

  if (is.null(model) || nchar(model) == 0) model <- defaults[[provider]]

  options(
    llmcoder.provider      = provider,
    llmcoder.api_key       = api_key %||% "",
    llmcoder.model         = model,
    llmcoder.context_lines = as.integer(context_lines),
    llmcoder.ollama_url    = ollama_url,
    llmcoder.custom_url    = custom_url
  )

  message(sprintf(
    "llmcoder configured: provider=%s, model=%s, context_lines=%d",
    provider, model, as.integer(context_lines)
  ))
  invisible(NULL)
}


#' Show the current LLMcoder configuration
#'
#' Returns (and prints) the active provider, model, API key (masked),
#' context-lines setting, and any provider-specific URLs.
#'
#' @return An object of class `"llmcoder_config"`: a named list with elements
#'   `provider`, `model`, `api_key`, `context_lines`, `ollama_url`, and
#'   `custom_url`.  The API key is masked for security.  When printed, it
#'   displays in a human-readable table.
#'
#' @examples
#' # Show current configuration (reads from option values)
#' llmcoder_config()
#'
#' # Capture the config as a list for programmatic use
#' cfg <- llmcoder_config()
#' cfg$provider
#' cfg$model
#'
#' @seealso [llmcoder_setup()]
#' @export
llmcoder_config <- function() {
  key_raw <- getOption("llmcoder.api_key", "")
  key_display <- if (nchar(key_raw) > 4)
    paste0(substr(key_raw, 1, 4), strrep("*", nchar(key_raw) - 4))
  else if (nchar(key_raw) > 0) "****"
  else "(not set)"

  cfg <- list(
    provider      = getOption("llmcoder.provider",      "(not set)"),
    model         = getOption("llmcoder.model",         "(not set)"),
    api_key       = key_display,
    context_lines = getOption("llmcoder.context_lines", 40L),
    ollama_url    = getOption("llmcoder.ollama_url",    "http://localhost:11434"),
    custom_url    = getOption("llmcoder.custom_url",    "")
  )
  class(cfg) <- "llmcoder_config"
  cfg
}

#' @export
print.llmcoder_config <- function(x, ...) {
  cat("llmcoder configuration:\n")
  cat("  Provider      :", x$provider,      "\n")
  cat("  Model         :", x$model,         "\n")
  cat("  API key       :", x$api_key,       "\n")
  cat("  Context lines :", x$context_lines, "\n")
  cat("  Ollama URL    :", x$ollama_url,    "\n")
  if (nchar(x$custom_url) > 0)
    cat("  Custom URL    :", x$custom_url,   "\n")
  invisible(x)
}

# Null-coalescing (also defined in gadgets.R — keep consistent)
`%||%` <- function(a, b) if (!is.null(a) && length(a) > 0 && nchar(as.character(a)[1]) > 0) a else b

# ============================================================
#  llmcoder — LLM API layer
# ============================================================

#' Call the configured LLM
#'
#' Unified dispatch function that reads provider, model, and credentials from
#' options (set via [llmcoder_setup()] or **Addins \→ LLMcoder Settings**) and
#' forwards the request to the appropriate backend.
#'
#' @param prompt       Character. The user-facing instruction.
#' @param system_prompt Character. The system-level instruction for the model.
#' @param context      Character or `NULL`. Surrounding R code sent as
#'   additional context (prepended to `prompt`).
#'
#' @return Character string containing the model's response text.
#'
#' @details
#' Supported providers:
#' \describe{
#'   \item{`"openai"`}{OpenAI Chat Completions API
#'     (`https://api.openai.com/v1`).}
#'   \item{`"anthropic"`}{Anthropic Messages API
#'     (`https://api.anthropic.com/v1/messages`).}
#'   \item{`"deepseek"`}{DeepSeek Chat API, OpenAI-compatible
#'     (`https://api.deepseek.com/v1`).}
#'   \item{`"ollama"`}{Local Ollama server (default
#'     `http://localhost:11434`).  No API key required.}
#'   \item{`"groq"`}{Groq Cloud API, OpenAI-compatible
#'     (`https://api.groq.com/openai/v1`).  Extremely fast inference.}
#'   \item{`"together"`}{Together AI API, OpenAI-compatible
#'     (`https://api.together.xyz/v1`).  Wide open-source model selection.}
#'   \item{`"openrouter"`}{OpenRouter API, OpenAI-compatible
#'     (`https://openrouter.ai/api/v1`).  Unified gateway to 100+ models.}
#'   \item{`"custom"`}{Any OpenAI-compatible server.  Set
#'     `llmcoder.custom_url` to the base URL (e.g.\
#'     `"http://localhost:1234/v1"` for LM Studio).}
#' }
#'
#' @examples
#' \dontrun{
#' llmcoder_setup("ollama", model = "llama3")
#' resp <- call_llm(
#'   prompt        = "Write R code to compute the mean of a numeric vector",
#'   system_prompt = "You are an R programming assistant.",
#'   context       = NULL
#' )
#' cat(resp)
#' }
#'
#' @keywords internal
call_llm <- function(prompt, system_prompt, context = NULL) {

  provider   <- getOption("llmcoder.provider",   "openai")
  api_key    <- getOption("llmcoder.api_key",    Sys.getenv("LLMCODER_API_KEY"))
  model      <- getOption("llmcoder.model",      default_model(provider))
  ollama_url <- getOption("llmcoder.ollama_url", "http://localhost:11434")

  if (is.null(provider) || !nzchar(trimws(provider))) {
    stop(
      "llmcoder.provider option is not set. ",
      "Run llmcoder_setup() or use the Settings addin to configure your provider.",
      call. = FALSE
    )
  }

  # Local / keyless providers
  local_providers <- c("ollama")
  if (!provider %in% local_providers &&
      (is.null(api_key) || nchar(api_key) == 0)) {
    stop(
      "No API key found. Run `llmcoder_setup()` or set the ",
      "LLMCODER_API_KEY environment variable.", call. = FALSE
    )
  }

  # Inject context
  full_prompt <- if (!is.null(context) && nchar(trimws(context)) > 0) {
    paste0(
      "Here is the surrounding R code for context:\n```r\n",
      context, "\n```\n\n", prompt
    )
  } else {
    prompt
  }

  switch(provider,
    openai = call_openai_compat(
      full_prompt, system_prompt, api_key, model,
      base_url = "https://api.openai.com/v1"),

    anthropic = call_anthropic(
      full_prompt, system_prompt, api_key, model),

    deepseek = call_openai_compat(
      full_prompt, system_prompt, api_key, model,
      base_url = "https://api.deepseek.com/v1"),

    ollama = call_ollama(
      full_prompt, system_prompt, model, base_url = ollama_url),

    groq = call_openai_compat(
      full_prompt, system_prompt, api_key, model,
      base_url = "https://api.groq.com/openai/v1"),

    together = call_openai_compat(
      full_prompt, system_prompt, api_key, model,
      base_url = "https://api.together.xyz/v1"),

    openrouter = call_openai_compat(
      full_prompt, system_prompt, api_key, model,
      base_url    = "https://openrouter.ai/api/v1",
      extra_hdrs  = list(
        "HTTP-Referer" = "https://github.com/ShiyangZheng/llmcoder",
        "X-Title"      = "llmcoder"
      )),

    custom = {
      custom_url <- getOption("llmcoder.custom_url", "")
      if (nchar(trimws(custom_url)) == 0)
        stop(
          "Set the 'llmcoder.custom_url' option (or use the Settings addin) ",
          "before using the 'custom' provider.", call. = FALSE
        )
      call_openai_compat(full_prompt, system_prompt, api_key, model,
                         base_url = custom_url)
    },

    stop(
      "Unknown provider: '", provider, "'. ",
      "Valid choices: openai, anthropic, deepseek, ollama, ",
      "groq, together, openrouter, custom.", call. = FALSE
    )
  )
}


# ---------- Provider implementations ------------------------------------------

#' Generic OpenAI-compatible chat completions call
#' @keywords internal
call_openai_compat <- function(prompt, system_prompt, api_key, model,
                                base_url, extra_hdrs = character()) {

  hdrs <- c(
    "Authorization" = paste("Bearer", api_key),
    "Content-Type"  = "application/json",
    unname(extra_hdrs)
  )

  body_data <- list(
    model       = model,
    messages    = list(
      list(role = "system", content = system_prompt),
      list(role = "user",   content = prompt)
    ),
    temperature = 0.2,
    max_tokens  = 2000
  )

  base <- httr2::request(paste0(trimws(base_url, "right"), "/chat/completions"))

  # Build headers using do.call (avoids non-standard-evaluation issues in pipeline)
  req <- do.call(httr2::req_headers, c(list(.req = base), hdrs))
  req <- httr2::req_body_json(req, data = body_data)
  req <- httr2::req_error(req, is_error = \(r) FALSE)
  req <- httr2::req_timeout(req, 120)

  resp <- httr2::req_perform(req)
  body <- httr2::resp_body_json(resp)

  if (!is.null(body$error)) {
    stop("API error (", httr2::resp_status(resp), "): ",
         body$error$message, call. = FALSE)
  }

  content <- body$choices[[1]]$message$content
  if (is.null(content) || !is.character(content) || length(content) == 0) {
    stop(
      "LLM returned empty content. ",
      "The model may have returned a tool/function call instead of text.",
      call. = FALSE
    )
  }
  content
}

#' Anthropic Messages API call
#' @keywords internal
call_anthropic <- function(prompt, system_prompt, api_key, model) {

  req <- httr2::request("https://api.anthropic.com/v1/messages") |>
    httr2::req_headers(
      "x-api-key"         = api_key,
      "anthropic-version" = "2023-06-01",
      "Content-Type"      = "application/json"
    ) |>
    httr2::req_body_json(data = list(
      model      = model,
      max_tokens = 2000,
      system     = system_prompt,
      messages   = list(list(role = "user", content = prompt))
    )) |>
    httr2::req_error(is_error = \(r) FALSE) |>
    httr2::req_timeout(120)

  resp <- httr2::req_perform(req)
  body <- httr2::resp_body_json(resp)

  if (!is.null(body$error)) {
    stop("Anthropic API error (", httr2::resp_status(resp), "): ",
         body$error$message, call. = FALSE)
  }
  body$content[[1]]$text
}

#' Ollama local API (uses the OpenAI-compatible /v1 endpoint, Ollama >= 0.1.24)
#'
#' No API key is required.  The Ollama server must be running locally;
#' start it with `ollama serve` in a terminal.
#' @keywords internal
call_ollama <- function(prompt, system_prompt, model, base_url) {
  call_openai_compat(
    prompt        = prompt,
    system_prompt = system_prompt,
    api_key       = "ollama",   # accepted but ignored by Ollama
    model         = model,
    base_url      = paste0(trimws(base_url, "right"), "/v1")
  )
}


# ---------- Helpers -----------------------------------------------------------

#' Default model name per provider
#'
#' Returns a sensible default model name when the user has not specified one
#' explicitly.
#'
#' @param provider Character. Provider identifier (see [call_llm()]).
#' @return Character string with the default model name.
#' @examples
#' \dontrun{
#' default_model("openai")
#' default_model("anthropic")
#' default_model("ollama")
#' }
#' @keywords internal
default_model <- function(provider) {
  switch(provider,
    openai     = "gpt-4o-mini",
    anthropic  = "claude-sonnet-4-20250514",
    deepseek   = "deepseek-chat",
    ollama     = "llama3",
    groq       = "llama-3.3-70b-versatile",
    together   = "meta-llama/Llama-3-70b-chat-hf",
    openrouter = "openai/gpt-4o-mini",
    custom     = "",
    "gpt-4o-mini"
  )
}


#' List models available on a running Ollama server
#'
#' Queries `GET /api/tags` on the local Ollama REST API and returns the names
#' of all installed models.  Useful for populating the model selector in the
#' Settings gadget.
#'
#' @param base_url Character. Ollama base URL.  Defaults to the value of
#'   `getOption("llmcoder.ollama_url", "http://localhost:11434")`.
#' @return Character vector of model tag names, or `NULL` if Ollama is not
#'   reachable.
#'
#' @details
#' Ollama must be running (`ollama serve`) before calling this function.
#' Models are installed with `ollama pull <model>` from the terminal.
#'
#' @examples
#' \dontrun{
#' ollama_list_models()
#' # [1] "llama3:latest"  "qwen2.5-coder:7b"  "mistral:latest"
#' }
#'
#' @export
ollama_list_models <- function(base_url = getOption("llmcoder.ollama_url",
                                                     "http://localhost:11434")) {
  url <- paste0(trimws(base_url, "right"), "/api/tags")
  tryCatch({
    req  <- httr2::request(url) |> httr2::req_timeout(5)
    resp <- httr2::req_perform(req)
    body <- httr2::resp_body_json(resp)
    vapply(body$models, `[[`, character(1), "name")
  }, error = function(e) {
    message("Could not reach Ollama at ", base_url,
            ".\nIs it running? Try: ollama serve")
    NULL
  })
}


# ---------- System prompts ----------------------------------------------------

#' System prompt for code generation
#' @keywords internal
build_system_prompt <- function() {
  paste0(
    "You are an expert R programmer assisting a researcher inside RStudio. ",
    "The user will provide a comment written in R (prefixed with #) describing ",
    "the code they want. Your task:\n",
    "1. Output ONLY raw R code \u2014 no markdown fences, no explanations, no preamble.\n",
    "2. The code must directly implement what the comment requests.\n",
    "3. Use tidyverse idioms (dplyr, ggplot2, tidyr, purrr) when appropriate.\n",
    "4. Keep variable names concise and meaningful.\n",
    "5. Add brief inline comments only where logic is non-obvious.\n",
    "6. If the comment is ambiguous, implement the most reasonable interpretation.\n",
    "7. Do NOT reproduce the original comment \u2014 only output the code.\n",
    "The user's field is psycholinguistics / applied linguistics. ",
    "Prefer lme4/lmerTest for mixed models, ggplot2 for visualisation, ",
    "and tidyverse for data wrangling."
  )
}

#' System prompt for code explanation
#'
#' Returns the system prompt instructing the LLM to write R comments
#' explaining the user's selected code.
#'
#' @return Character string: the system prompt sent to the LLM for the
#'   explain workflow.
#' @examples
#' \dontrun{
#' build_explain_prompt()
#' }
#' @keywords internal
build_explain_prompt <- function() {
  paste0(
    "You are an expert R programmer. The user will provide R code. ",
    "Write a clear, concise explanation as R comments (each line prefixed with # ). ",
    "Output ONLY the comment lines \u2014 no code, no markdown. ",
    "Focus on what the code does and why, not how R syntax works."
  )
}

#' System prompt for error fixing
#'
#' Returns the system prompt instructing the LLM to diagnose an R error and
#' produce corrected code.
#'
#' @return Character string: the system prompt sent to the LLM for the
#'   fix workflow.
#' @examples
#' \dontrun{
#' build_fix_prompt()
#' }
#' @keywords internal
build_fix_prompt <- function() {
  paste0(
    "You are an expert R programmer debugging code inside RStudio. ",
    "The user will provide:\n",
    "  (a) An R error message from the console.\n",
    "  (b) The R source code that produced it.\n\n",
    "Your task:\n",
    "1. Diagnose the root cause of the error.\n",
    "2. Output the COMPLETE corrected source code \u2014 not just the changed lines.\n",
    "3. Mark each changed line with a short comment: # FIX: <reason>.\n",
    "4. Output ONLY raw R code \u2014 no markdown fences, no preamble, no explanation outside comments.\n",
    "5. Preserve all existing formatting and unrelated comments.\n",
    "6. If there are multiple plausible fixes, choose the most conservative one.\n",
    "The user's field is psycholinguistics / applied linguistics."
  )
}


# ============================================================
#  llmcoder — Multi-turn message history API
# ============================================================

#' Call the configured LLM with full message history
#'
#' Like [call_llm()] but accepts a `messages` list that preserves the full
#' conversation context across multiple turns.  This is the engine behind
#' [addin_chat_panel()].
#'
#' @param messages List of messages, each element being
#'   `list(role = "system"|"user"|"assistant", content = "...")`.
#'   The **last** message must have `role = "user"`.
#' @param system_prompt_override  Optional character string.  If supplied,
#'   it **replaces** the system role message in `messages` entirely
#'   (useful for swapping in a custom persona at call-time).
#'
#' @return Character string: the model's response text.
#'
#' @keywords internal
call_llm_history <- function(messages, system_prompt_override = NULL) {

  provider   <- getOption("llmcoder.provider",   "openai")
  api_key    <- getOption("llmcoder.api_key",    Sys.getenv("LLMCODER_API_KEY"))
  model      <- getOption("llmcoder.model",       default_model(provider))
  ollama_url <- getOption("llmcoder.ollama_url", "http://localhost:11434")

  # Guard against NULL / empty provider (R switch silently returns NULL for these)
  if (is.null(provider) || !nzchar(trimws(provider))) {
    stop(
      "llmcoder.provider option is not set. ",
      "Run llmcoder_setup() or use the Settings addin to configure your provider.",
      call. = FALSE
    )
  }

  local_providers <- c("ollama")
  if (!provider %in% local_providers &&
      (is.null(api_key) || nchar(api_key) == 0)) {
    stop(
      "No API key found. Run `llmcoder_setup()` or set the ",
      "LLMCODER_API_KEY environment variable.", call. = FALSE
    )
  }

  # Optionally replace system message
  msgs <- messages
  if (!is.null(system_prompt_override)) {
    sys_idx <- which(vapply(msgs, `[[`, logical(1), "role") == "system")
    if (length(sys_idx) > 0) {
      msgs[[sys_idx[[1]]]] <- list(role = "system", content = system_prompt_override)
    } else {
      msgs <- c(list(list(role = "system", content = system_prompt_override)), msgs)
    }
  }

  switch(provider,
    openai = call_openai_compat_history(
      msgs, api_key, model, base_url = "https://api.openai.com/v1"),

    anthropic = call_anthropic_history(
      msgs, api_key, model),

    deepseek = call_openai_compat_history(
      msgs, api_key, model, base_url = "https://api.deepseek.com/v1"),

    ollama = call_ollama_history(
      msgs, model, base_url = ollama_url),

    groq = call_openai_compat_history(
      msgs, api_key, model, base_url = "https://api.groq.com/openai/v1"),

    together = call_openai_compat_history(
      msgs, api_key, model, base_url = "https://api.together.xyz/v1"),

    openrouter = call_openai_compat_history(
      msgs, api_key, model,
      base_url    = "https://openrouter.ai/api/v1",
      extra_hdrs  = list(
        "HTTP-Referer" = "https://github.com/ShiyangZheng/llmcoder",
        "X-Title"      = "llmcoder"
      )),

    custom = {
      custom_url <- getOption("llmcoder.custom_url", "")
      if (nchar(trimws(custom_url)) == 0)
        stop("Set the 'llmcoder.custom_url' option before using the 'custom' provider.",
             call. = FALSE)
      call_openai_compat_history(msgs, api_key, model, base_url = custom_url)
    },

    stop("Unknown provider: '", provider, "'. ",
         "Valid choices: openai, anthropic, deepseek, ollama, ",
         "groq, together, openrouter, custom.", call. = FALSE)
  )
}


#' OpenAI-compatible multi-turn call
#'
#' @param messages   List of message objects.
#' @param api_key    API key.
#' @param model      Model name.
#' @param base_url   API base URL.
#' @param extra_hdrs Named character vector of extra headers.
#' @keywords internal
call_openai_compat_history <- function(messages, api_key, model,
                                       base_url,
                                       extra_hdrs = character()) {

  hdrs <- c(
    "Authorization" = paste("Bearer", api_key),
    "Content-Type"  = "application/json",
    unname(extra_hdrs)
  )

  body_data <- list(
    model       = model,
    messages    = messages,
    temperature = 0.7,
    max_tokens  = 2000
  )

  base <- httr2::request(
    paste0(trimws(base_url, "right"), "/chat/completions")
  )

  req <- do.call(httr2::req_headers, c(list(.req = base), hdrs))
  req <- httr2::req_body_json(req, data = body_data)
  req <- httr2::req_error(req, is_error = \(r) FALSE)
  req <- httr2::req_timeout(req, 120)

  resp <- httr2::req_perform(req)
  body <- httr2::resp_body_json(resp)

  if (!is.null(body$error)) {
    stop("API error (", httr2::resp_status(resp), "): ",
         body$error$message, call. = FALSE)
  }

  content <- body$choices[[1]]$message$content

  # Defensive: ensure we got a character string, not a function_call or NULL
  if (is.null(content) || !is.character(content) || length(content) == 0) {
    finish_reason <- body$choices[[1]]$finish_reason %||% "(unknown)"
    stop(
      "LLM returned empty content (finish_reason = '", finish_reason, "'). ",
      "The model may have returned a tool/function call instead of text. ",
      "Check that 'tools' or 'function_call' parameters are not set.",
      call. = FALSE
    )
  }
  content
}


#' Anthropic Messages API with multi-turn history
#'
#' Extracts the system role from `messages` and moves it to the top-level
#' \code{system} field, as required by the Anthropic API.  Only
#' \code{user} and \code{assistant} roles are allowed.
#'
#' @param messages  List of message objects.
#' @param api_key   API key.
#' @param model     Model name.
#' @keywords internal
call_anthropic_history <- function(messages, api_key, model) {

  # Pull system out
  sys_content <- ""
  non_sys <- messages

  sys_idx <- which(vapply(messages, `[[`, logical(1), "role") == "system")
  if (length(sys_idx) > 0) {
    sys_content <- messages[[sys_idx[[1]]]]$content
    non_sys <- messages[-sys_idx[[1]]]
  }

  # Only user/assistant roles are permitted
  allowed_roles <- c("user", "assistant")
  anth_messages <- lapply(non_sys, function(m) {
    if (!(m$role %in% allowed_roles)) {
      list(role = "user", content = paste0("[", m$role, " message]: ", m$content))
    } else {
      m
    }
  })

  req <- httr2::request("https://api.anthropic.com/v1/messages") |>
    httr2::req_headers(
      "x-api-key"          = api_key,
      "anthropic-version"  = "2023-06-01",
      "Content-Type"       = "application/json"
    ) |>
    httr2::req_body_json(data = list(
      model      = model,
      max_tokens = 2000,
      system     = sys_content,
      messages   = anth_messages
    )) |>
    httr2::req_error(is_error = \(r) FALSE) |>
    httr2::req_timeout(120)

  resp <- httr2::req_perform(req)
  body <- httr2::resp_body_json(resp)

  if (!is.null(body$error)) {
    stop("Anthropic API error (", httr2::resp_status(resp), "): ",
         body$error$message, call. = FALSE)
  }
  body$content[[1]]$text
}


#' Ollama multi-turn (OpenAI-compatible endpoint)
#' @keywords internal
call_ollama_history <- function(messages, model, base_url) {
  call_openai_compat_history(
    messages   = messages,
    api_key    = "ollama",
    model      = model,
    base_url   = paste0(trimws(base_url, "right"), "/v1")
  )
}


# ---------- Infix helpers ----------------------------------------------------

#' Null-coalescing operator: returns x if it is non-NULL, otherwise y
#' @keywords internal
`%||%` <- function(x, y) {
  if (is.null(x)) y else x
}

# ============================================================
#  llmcoder — RStudio addin entry points
# ============================================================

#' Generate R code from a comment (silent insert)
#'
#' Places the cursor on a line beginning with `#`, then triggers this addin
#' (default shortcut: `Ctrl+Shift+G` on Windows/Linux, `Cmd+Shift+G` on macOS).
#' The LLM reads the comment text and the surrounding code context, then inserts
#' the generated R code on the line immediately below the comment.
#'
#' @details
#' The addin extracts the text of the comment at the cursor position and up to
#' `getOption("llmcoder.context_lines", 40L)` lines of preceding code as
#' context.  The provider, model, and API key are taken from options set by
#' [llmcoder_setup()] or the **LLMcoder Settings** addin.
#'
#' No dialog is shown; code is inserted immediately.  Use
#' [addin_generate_with_preview()] if you prefer to review the output first.
#'
#' @return Invisible `NULL` (called for side-effects).
#' @seealso [addin_generate_with_preview()], [llmcoder_setup()]
#' @export
addin_generate_from_comment <- function() {

  ctx <- safe_get_ctx(); if (is.null(ctx)) return(invisible(NULL))

  info <- tryCatch(extract_comment_at_cursor(ctx), error = function(e) {
    rstudioapi::showDialog("LLMcoder", conditionMessage(e)); NULL
  })
  if (is.null(info)) return(invisible(NULL))

  n_ctx <- getOption("llmcoder.context_lines", 40L)
  context_code <- gather_context(ctx, info$row, n = n_ctx)

  notify(paste0('Generating: "', info$comment, '" ...'))

  code <- safe_call_llm(
    prompt        = paste0("Write R code that: ", info$comment),
    system_prompt = build_system_prompt(),
    context       = context_code
  )
  if (is.null(code)) return(invisible(NULL))

  insert_after_row(clean_code_output(code), info$row, indent = info$indent)
  notify("Done.")
  invisible(NULL)
}


#' Generate R code with an editable preview dialog
#'
#' Same as [addin_generate_from_comment()] but opens a Shiny gadget so you can
#' review and optionally edit the generated code before it is inserted into the
#' editor.  Recommended shortcut: `Ctrl+Shift+P` / `Cmd+Shift+P`.
#'
#' @details
#' The preview dialog shows the generated code in an editable text area.
#' Click **Insert** to place it in the editor, or close the dialog to discard
#' the result.
#'
#' @return Invisible `NULL` (called for side-effects).
#' @seealso [addin_generate_from_comment()], [llmcoder_setup()]
#' @export
addin_generate_with_preview <- function() {

  ctx <- safe_get_ctx(); if (is.null(ctx)) return(invisible(NULL))

  info <- tryCatch(extract_comment_at_cursor(ctx), error = function(e) {
    rstudioapi::showDialog("LLMcoder", conditionMessage(e)); NULL
  })
  if (is.null(info)) return(invisible(NULL))

  n_ctx <- getOption("llmcoder.context_lines", 40L)
  context_code <- gather_context(ctx, info$row, n = n_ctx)

  notify(paste0('Calling LLM for: "', info$comment, '" ...'))

  code <- safe_call_llm(
    prompt        = paste0("Write R code that: ", info$comment),
    system_prompt = build_system_prompt(),
    context       = context_code
  )
  if (is.null(code)) return(invisible(NULL))

  preview_gadget(
    comment = info$comment,
    code    = clean_code_output(code),
    row     = info$row,
    indent  = info$indent,
    ctx_id  = ctx$id
  )
  invisible(NULL)
}


#' Fix the last console error automatically
#'
#' After running code that produces an error in the R console, trigger this
#' addin (recommended shortcut: `Ctrl+Shift+F` / `Cmd+Shift+F`).
#'
#' @details
#' The addin attempts to recover the most recent error message using several
#' strategies, in order of priority:
#'
#' \enumerate{
#'   \item The `rlang` last-error store (`rlang::last_error()`), which captures
#'         errors thrown by \pkg{rlang}-aware packages and the tidyverse.
#'   \item Base R's `.Last.error` binding (set whenever an unhandled condition
#'         reaches the top level).
#'   \item The `.Last.error.trace` character vector written by some versions of
#'         \pkg{rlang}.
#' }
#'
#' The complete source file currently open in the editor is also sent to the LLM
#' as context.  The LLM returns the entire corrected file, with changed lines
#' annotated as `# FIX: <reason>`.  A diff-style preview dialog lets you review
#' and edit the fix before applying it.
#'
#' **Workflow**
#' 1. Run code — error appears in console.
#' 2. Trigger this addin.
#' 3. Review the fix in the preview dialog → click **Apply Fix**.
#'
#' If no recent error is detected, a dialog explains the possible reasons and
#' suggests using [addin_fix_selected_error()] instead.
#'
#' @return Invisible `NULL` (called for side-effects).
#' @seealso [addin_fix_selected_error()], [llmcoder_setup()]
#' @export
addin_fix_console_error <- function() {

  # ---- 1. Grab the last error — try multiple sources -------------------
  last_err <- .collect_last_error()

  if (is.null(last_err) || nchar(trimws(last_err)) == 0) {
    rstudioapi::showDialog(
      "LLMcoder \u2014 Fix Error",
      paste0(
        "No recent error found.\n\n",
        "Possible reasons:\n",
        "  \u2022 The code was sourced inside a tryCatch() that swallowed the error.\n",
        "  \u2022 You are using a session where .Last.error is not set.\n\n",
        "Alternative: copy the error text from the console, paste it somewhere\n",
        "in the editor, select it, and use 'Fix Selected Error Text' instead."
      )
    )
    return(invisible(NULL))
  }

  # ---- 2. Get the current source file ----------------------------------
  ctx <- safe_get_ctx(); if (is.null(ctx)) return(invisible(NULL))

  source_code <- paste(ctx$contents, collapse = "\n")

  if (nchar(trimws(source_code)) == 0) {
    rstudioapi::showDialog(
      "LLMcoder \u2014 Fix Error",
      "The current editor appears to be empty.\nOpen the file that caused the error."
    )
    return(invisible(NULL))
  }

  # ---- 3. Call LLM -----------------------------------------------------
  notify(paste0("Diagnosing error: ", substr(last_err, 1, 80), " ..."))

  prompt <- paste0(
    "ERROR MESSAGE:\n", last_err,
    "\n\nSOURCE CODE:\n```r\n", source_code, "\n```\n\n",
    "Fix the error. Return the complete corrected source."
  )

  fixed_code <- safe_call_llm(
    prompt        = prompt,
    system_prompt = build_fix_prompt(),
    context       = NULL
  )
  if (is.null(fixed_code)) return(invisible(NULL))

  # ---- 4. Show preview + apply -----------------------------------------
  fix_preview_gadget(
    error_msg  = last_err,
    original   = source_code,
    fixed      = clean_code_output(fixed_code),
    ctx        = ctx
  )

  invisible(NULL)
}


#' Fix an error by selecting its text
#'
#' Select the error message text in the editor (or paste it into a temporary
#' comment), then trigger this addin.  The addin pairs the selected text with
#' the complete source file currently open in the editor and asks the LLM for a
#' fix, displaying the result in a review dialog.
#'
#' @details
#' This addin is the recommended fallback when [addin_fix_console_error()] does
#' not detect an error automatically (e.g., because the error occurred inside a
#' `tryCatch()` block or in a separate R process).
#'
#' **Workflow**
#' 1. Copy the error message from the console.
#' 2. Paste it anywhere in the source file, or simply select it in the console
#'    output if your terminal supports that.
#' 3. Select the error text in the editor.
#' 4. Trigger this addin.
#' 5. Review and apply the suggested fix.
#'
#' @return Invisible `NULL` (called for side-effects).
#' @seealso [addin_fix_console_error()], [llmcoder_setup()]
#' @export
addin_fix_selected_error <- function() {

  ctx <- safe_get_ctx(); if (is.null(ctx)) return(invisible(NULL))
  sel <- ctx$selection[[1]]$text

  if (nchar(trimws(sel)) == 0) {
    rstudioapi::showDialog(
      "LLMcoder \u2014 Fix Selected Error",
      "Select the error message text first, then trigger this addin."
    )
    return(invisible(NULL))
  }

  source_code <- paste(ctx$contents, collapse = "\n")

  notify("Diagnosing selected error ...")

  prompt <- paste0(
    "ERROR MESSAGE:\n", sel,
    "\n\nSOURCE CODE:\n```r\n", source_code, "\n```\n\n",
    "Fix the error. Return the complete corrected source."
  )

  fixed_code <- safe_call_llm(
    prompt        = prompt,
    system_prompt = build_fix_prompt(),
    context       = NULL
  )
  if (is.null(fixed_code)) return(invisible(NULL))

  fix_preview_gadget(
    error_msg = sel,
    original  = source_code,
    fixed     = clean_code_output(fixed_code),
    ctx       = ctx
  )
  invisible(NULL)
}


#' Explain selected R code as inline comments
#'
#' Select a block of R code in the editor, then trigger this addin
#' (recommended shortcut: `Ctrl+Shift+E` / `Cmd+Shift+E`).  An explanation is
#' inserted as `#` comment lines immediately **above** the selected code block.
#'
#' @details
#' The LLM receives the selected code and is instructed to produce a concise,
#' human-readable explanation — focusing on *what* the code does and *why*,
#' not on basic R syntax.  Every output line is prefixed with `# ` so the
#' explanation is valid R that can be left in the source file.
#'
#' @return Invisible `NULL` (called for side-effects).
#' @seealso [addin_generate_from_comment()], [llmcoder_setup()]
#' @export
addin_explain_code <- function() {

  ctx <- safe_get_ctx(); if (is.null(ctx)) return(invisible(NULL))
  sel <- ctx$selection[[1]]

  if (nchar(trimws(sel$text)) == 0) {
    rstudioapi::showDialog("LLMcoder",
      "No code selected. Select the R code you want explained, then try again.")
    return(invisible(NULL))
  }

  notify("Generating explanation ...")

  explanation <- safe_call_llm(
    prompt        = paste0("Explain this R code:\n```r\n", sel$text, "\n```"),
    system_prompt = build_explain_prompt(),
    context       = NULL
  )
  if (is.null(explanation)) return(invisible(NULL))

  # Ensure every line starts with #
  lines <- stringr::str_split(explanation, "\n")[[1]]
  lines <- ifelse(stringr::str_starts(lines, "#"), lines, paste0("# ", lines))
  comment_block <- paste(lines, collapse = "\n")

  rstudioapi::insertText(
    location = rstudioapi::document_position(sel$range$start[["row"]], 1L),
    text     = paste0(comment_block, "\n"),
    id       = ctx$id
  )
  notify("Explanation inserted.")
  invisible(NULL)
}


#' Open the LLMcoder settings dialog
#'
#' Launches an interactive Shiny gadget that lets you configure the LLM
#' provider, model, API key, Ollama URL (for local models), custom base URL
#' (for LM Studio / vLLM / llama.cpp), and context-window size.  Settings can
#' optionally be persisted to `~/.Rprofile` so they survive R restarts.
#'
#' @return Invisible `NULL` (called for side-effects).
#' @seealso [llmcoder_setup()], [llmcoder_config()]
#' @export
addin_settings <- function() settings_gadget()


# ---------- Internal helpers --------------------------------------------------

#' Strip markdown code fences from LLM output
#'
#' Strips common markdown code fences from LLM output so the raw code can be
#' inserted into the editor.
#'
#' @param code Character string returned by an LLM, possibly wrapped in
#'   ` ```r` code fences.
#' @return Character string with fences removed.  If no fences are found, the
#'   input is returned as-is.
#' @examples
#' \dontrun{
#' raw <- "\n```r\nx <- mean(1:10)\nprint(x)\n```\n"
#' clean_code_output(raw)
#' clean_code_output("no fences here")
#' }
#' @keywords internal
clean_code_output <- function(code) {
  code <- stringr::str_trim(code)
  code <- stringr::str_replace(code, "^```[rR]?\\s*\n?", "")
  code <- stringr::str_replace(code, "\n?```\\s*$", "")
  stringr::str_trim(code)
}

#' Safely obtain the active editor context
#' @keywords internal
safe_get_ctx <- function() {
  tryCatch(get_editor_ctx(), error = function(e) {
    rstudioapi::showDialog("LLMcoder Error", conditionMessage(e))
    NULL
  })
}

#' Safely call the LLM, catching API errors
#' @keywords internal
safe_call_llm <- function(prompt, system_prompt, context) {
  tryCatch(
    call_llm(prompt = prompt, system_prompt = system_prompt, context = context),
    error = function(e) {
      rstudioapi::showDialog("LLMcoder API Error", conditionMessage(e))
      NULL
    }
  )
}

#' Collect the most recent R error message using multiple strategies
#'
#' Tries, in order: rlang::last_error(), .Last.error (base R),
#' .Last.error condition message.
#' Returns NULL if nothing is found.
#' @keywords internal
.collect_last_error <- function() {

  # Strategy 1: rlang last error (most reliable for tidyverse code)
  err <- tryCatch({
    e <- rlang::last_error()
    if (!is.null(e)) {
      paste0(
        conditionMessage(e),
        if (!is.null(e$trace)) paste0("\n\nTraceback:\n", format(e$trace)) else ""
      )
    } else {
      NULL
    }
  }, error = function(e2) NULL)

  if (!is.null(err) && nchar(trimws(err)) > 0) return(err)

  # Strategy 2: base R .Last.error as character
  # Use get() to avoid R CMD check "no visible binding" note
  last_err_obj <- tryCatch(get(".Last.error", envir = baseenv()),
                            error = function(e) NULL)

  err <- tryCatch(as.character(last_err_obj), error = function(e) NULL)
  if (!is.null(err) && nchar(trimws(err)) > 0) return(err)

  # Strategy 3: conditionMessage on .Last.error
  err <- tryCatch(conditionMessage(last_err_obj), error = function(e) NULL)
  if (!is.null(err) && nchar(trimws(err)) > 0) return(err)

  NULL
}

# ============================================================
#  llmcoder — Session Context collection
# ============================================================

#' @importFrom utils getFromNamespace
NULL

#' Capture a human-readable report of the current R session state
#'
#' @description
#' `session_context_report()` collects and formats the following information
#' from the current R session:
#' \itemize{
#'   \item R version and operating system.
#'   \item Loaded add-on packages (non-base).
#'   \item Global environment objects grouped by class.
#'   \item Contents of the active source editor (via \pkg{rstudioapi}).
#'   \item Console command history (via \pkg{rstudioapi}; falls back to
#'         `~/.Rhistory`).
#' }
#'
#' This report is primarily used internally to populate the system prompt
#' sent to the LLM in the [addin_chat_panel()] gadget, so the model has
#' full awareness of the analyst's working environment.
#'
#' @param max_objs   Maximum number of global objects to list per class group
#'                   (default 20).
#' @param max_hist   Maximum number of console history lines to include
#'                   (default 30).
#' @param quiet      If `TRUE`, suppress the `message()` emitted when
#'                   RStudio API is unavailable (default `FALSE`).
#' @return Character string. A multi-section report ready to embed in a
#'   system prompt.
#'
#' @examples
#' \dontrun{
#' report <- session_context_report()
#' cat(report)
#' }
#'
#' @export
session_context_report <- function(max_objs = 20L, max_hist = 30L,
                                   quiet = FALSE) {
  sections <- character(0)

  # ---- 1. R version + OS -------------------------------------------------
  sections <- c(sections, "## R Session Info",
    paste0("R ", R.version.string, " (", R.version$platform, ")"))

  # ---- 2. Loaded packages ------------------------------------------------
  pkgs <- sort(setdiff(.packages(), c("datasets", "utils", "grDevices",
                                      "graphics", "stats", "methods", "base")))
  if (length(pkgs)) {
    sections <- c(sections, "## Loaded Packages",
                  paste0("- ", pkgs, collapse = "\n"))
  } else {
    sections <- c(sections, "## Loaded Packages", "(none)")
  }

  # ---- 3. Global environment objects ------------------------------------
  objs <- ls(envir = .GlobalEnv, all.names = FALSE)
  if (length(objs) > 0) {
    objs_head <- if (length(objs) > max_objs) {
      c(utils::head(objs, max_objs), paste0("... and ", length(objs) - max_objs,
                                     " more (truncated)"))
    } else {
      objs
    }
    sections <- c(sections, "## Global Objects",
                  paste0("- `", objs_head, "`", collapse = "\n"))
  } else {
    sections <- c(sections, "## Global Objects", "(empty)")
  }

  # ---- 4. Source editor contents ----------------------------------------
  if (rstudioapi::isAvailable()) {
    src <- tryCatch({
      ctx <- rstudioapi::getSourceEditorContext()
      if (length(ctx$contents) && nchar(trimws(paste(ctx$contents, collapse = ""))) > 0) {
        paste0("File: ", ctx$path, "\n",
               paste(ctx$contents, collapse = "\n"))
      } else {
        "(no active file)"
      }
    }, error = function(e) "(unavailable)")

    sections <- c(sections, "## Source Editor", src)
  } else {
    if (!quiet) message("rstudioapi not available for source context.")
    sections <- c(sections, "## Source Editor", "(not available)")
  }

  # ---- 5. Console history -----------------------------------------------
  hist_lines <- .get_console_history(max_hist)
  if (length(hist_lines)) {
    sections <- c(sections, "## Console History (recent commands)",
                  paste0("> ", hist_lines, collapse = "\n"))
  } else {
    sections <- c(sections, "## Console History", "(no history available)")
  }

  paste(sections, collapse = "\n\n")
}


#' Build a session-context system-prompt block
#'
#' Convenience wrapper around [session_context_report()] that wraps the report
#' in a descriptive header so the LLM can distinguish it from user content.
#'
#' @param ...  Passed to [session_context_report()].
#' @return Character string suitable for prepending to a system prompt.
#'
#' @examples
#' \dontrun{
#' ctx_prompt <- session_context_prompt()
#' }
#'
#' @export
session_context_prompt <- function(...) {
  paste0(
    "Below is the current R session state. Use this context to give ",
    "accurate, environment-aware responses:\n\n",
    session_context_report(...)
  )
}


# ---------- Helpers -----------------------------------------------------------

#' Console history from rstudioapi with ~/.Rhistory fallback
#' @keywords internal
.get_console_history <- function(max_hist = 30L) {
  # Strategy 1: rstudioapi
  if (rstudioapi::isAvailable()) {
    tryCatch({
      f <- getFromNamespace("getConsoleHistory", "rstudioapi")
      h <- f()
      return(utils::tail(h, max_hist))
    }, error = function(e) NULL)
  }

  # Strategy 2: ~/.Rhistory
  rh <- path.expand("~/.Rhistory")
  if (file.exists(rh)) {
    tryCatch({
      lines <- readLines(rh, warn = FALSE)
      lines <- lines[nchar(trimws(lines)) > 0]
      return(utils::tail(lines, max_hist))
    }, error = function(e) NULL)
  }

  character(0)
}

# ============================================================
#  llmcoder - Chat Panel (multi-turn Shiny gadget)
# ============================================================

# ---------- CSS ---------------------------------------------------------------

#' @importFrom htmltools HTML
#' @importFrom shiny dialogViewer
#' @importFrom jsonlite fromJSON
#' @importFrom stringi
#'   stri_replace_all_fixed stri_replace_all_regex stri_opts_regex
#'   stri_split_regex stri_detect_regex stri_startswith_fixed stri_trans_toupper
NULL

utils::globalVariables("tags")

chat_css <- function() {
  "
  /* ---- Layout ---- */
  html, body {
    margin: 0;
    padding: 0;
    height: 100%;
    overflow: hidden;
    box-sizing: border-box;
  }

  .chat-layout {
    display: flex;
    flex-direction: row;
    height: 100%;       /* inherit from body/iframe to avoid 100vh overflow */
    width: 100%;
    overflow: hidden;
    box-sizing: border-box;
  }

  /* ---- Sidebar ---- */
  .chat-sidebar {
    width: 230px;
    min-width: 230px;
    background: #f8f9fa;
    border-right: 1px solid #dee2e6;
    display: flex;
    flex-direction: column;
    padding: 14px 12px;
    gap: 14px;
    overflow-y: auto;
  }

  .sidebar-logo {
    font-size: 14px;
    font-weight: 700;
    color: #2c3e50;
    padding-bottom: 6px;
    border-bottom: 1px solid #dee2e6;
  }

  .sidebar-section { display: flex; flex-direction: column; gap: 4px; }

  .sidebar-label {
    font-size: 11px;
    font-weight: 600;
    color: #868e96;
    text-transform: uppercase;
    letter-spacing: .5px;
    margin-bottom: 2px;
  }

  .sidebar-btn {
    display: flex;
    align-items: center;
    gap: 7px;
    padding: 6px 10px;
    border: 1px solid #dee2e6;
    border-radius: 6px;
    background: white;
    font-size: 13px;
    cursor: pointer;
    color: #495057;
    transition: background .15s;
    text-align: left;
    width: 100%;
  }

  .sidebar-btn:hover { background: #e9ecef; }
  .sidebar-btn.active {
    background: #228be6;
    color: white;
    border-color: #1c7cd6;
  }

  /* Toggle switch */
  .toggle-row {
    display: flex;
    align-items: center;
    justify-content: space-between;
    font-size: 13px;
    color: #495057;
  }

  .toggle-switch {
    position: relative; width: 36px; height: 20px;
    flex-shrink: 0;
  }

  .toggle-switch input { opacity: 0; width: 0; height: 0; }

  .toggle-slider {
    position: absolute; cursor: pointer;
    inset: 0;
    background: #adb5bd;
    border-radius: 20px;
    transition: .2s;
  }

  .toggle-slider::before {
    content: '';
    position: absolute;
    height: 14px; width: 14px;
    left: 3px; bottom: 3px;
    background: white;
    border-radius: 50%;
    transition: .2s;
  }

  .toggle-switch input:checked + .toggle-slider { background: #228be6; }
  .toggle-switch input:checked + .toggle-slider::before {
    transform: translateX(16px);
  }

  /* ---- Chat area ---- */
  .chat-main {
    flex: 1;
    display: flex;
    flex-direction: column;
    overflow: hidden;
    min-width: 0;
  }

  /* Scrollable message region -- sits between sidebar and fixed input */
  .msg-scroll-area {
    flex: 1;
    overflow-y: auto;
    min-height: 0;
  }

  .msg-container {
    padding: 16px 20px;
    display: flex;
    flex-direction: column;
    gap: 14px;
    scroll-behavior: smooth;
  }

  /* ---- Messages ---- */
  .msg {
    display: flex;
    gap: 10px;
    max-width: 100%;
    animation: msgIn .2s ease;
  }

  @keyframes msgIn {
    from { opacity: 0; transform: translateY(6px); }
    to   { opacity: 1; transform: translateY(0); }
  }

  .msg.user  { flex-direction: row-reverse; }
  .msg.error { flex-direction: row; }

  .msg-avatar {
    width: 30px; height: 30px;
    border-radius: 50%;
    display: flex; align-items: center; justify-content: center;
    font-size: 12px;
    flex-shrink: 0;
    font-weight: 700;
  }

  .msg.user .msg-avatar   { background: #e7f5ff; color: #1971c2; }
  .msg.assistant .msg-avatar { background: #f3f0ff; color: #6741d9; }
  .msg.error .msg-avatar { background: #fff5f5; color: #c92a2a; }

  .msg-body {
    max-width: 76%;
    background: white;
    border: 1px solid #dee2e6;
    border-radius: 12px;
    padding: 10px 14px;
    font-size: 14px;
    line-height: 1.6;
    color: #212529;
    word-break: break-word;
  }

  .msg.user .msg-body {
    background: #228be6;
    color: white;
    border-color: #1c7cd6;
    border-bottom-right-radius: 4px;
  }

  .msg.error .msg-body {
    background: #fff5f5;
    color: #c92a2a;
    border-color: #fcc2c2;
  }

  .msg.assistant .msg-body {
    border-bottom-left-radius: 4px;
  }

  /* Code blocks inside messages */
  .msg-body pre {
    background: #f8f9fa;
    border: 1px solid #dee2e6;
    border-radius: 6px;
    padding: 10px 12px;
    overflow-x: auto;
    font-size: 13px;
    font-family: 'Courier New', Courier, monospace;
    margin: 8px 0;
    position: relative;
  }

  .msg.user pre { background: rgba(255,255,255,.15); border-color: rgba(255,255,255,.3); }

  .code-header {
    display: flex;
    align-items: center;
    justify-content: space-between;
    padding: 4px 0 6px;
    border-bottom: 1px solid #dee2e6;
    margin-bottom: 6px;
  }

  .msg.user .code-header { border-color: rgba(255,255,255,.25); }

  .code-lang {
    font-size: 11px;
    font-weight: 600;
    color: #868e96;
    text-transform: uppercase;
  }

  .btn-run-code {
    font-size: 12px;
    padding: 3px 10px;
    background: #228be6;
    color: white;
    border: none;
    border-radius: 4px;
    cursor: pointer;
    transition: background .15s;
    margin-left: 4px;
  }

  .btn-run-code:hover { background: #1971c2; }

  .btn-insert-code {
    font-size: 12px;
    padding: 3px 10px;
    background: #37b24d;
    color: white;
    border: none;
    border-radius: 4px;
    cursor: pointer;
    transition: background .15s;
    margin-left: 4px;
  }

  .btn-insert-code:hover { background: #2f9e44; }

  .msg-body code {
    font-family: 'Courier New', Courier, monospace;
    font-size: 13px;
  }

  .msg-body p { margin: 0 0 8px; }
  .msg-body p:last-child { margin-bottom: 0; }
  .msg-body ul, .msg-body ol { margin: 4px 0; padding-left: 20px; }
  .msg-body strong { font-weight: 600; }
  .msg-body em { font-style: italic; }

  /* ---- Thinking indicator ---- */
  .thinking-wrap {
    display: flex;
    gap: 10px;
    padding: 0 20px 10px;
    animation: msgIn .2s ease;
  }

  .thinking-dots {
    display: flex; gap: 4px; align-items: center;
    padding: 10px 14px;
    background: white;
    border: 1px solid #dee2e6;
    border-radius: 12px;
    border-bottom-left-radius: 4px;
  }

  .dot {
    width: 6px; height: 6px;
    background: #868e96;
    border-radius: 50%;
    animation: bounce 1.2s infinite;
  }

  .dot:nth-child(2) { animation-delay: .2s; }
  .dot:nth-child(3) { animation-delay: .4s; }

  @keyframes bounce {
    0%, 80%, 100% { transform: translateY(0); }
    40%           { transform: translateY(-6px); }
  }

  /* ---- Input area ---- */
  .chat-input-wrap {
    display: flex;
    align-items: flex-end;
    gap: 8px;
    padding: 10px 14px;
    border-top: 1px solid #dee2e6;
    background: white;
    flex-shrink: 0;
  }

  #chat_input {
    flex: 1;
    min-height: 42px;
    max-height: 160px;
    padding: 10px 14px;
    border: 1px solid #ced4da;
    border-radius: 8px;
    font-size: 14px;
    font-family: inherit;
    resize: none;
    line-height: 1.5;
    overflow-y: auto;
    outline: none;
    transition: border-color .15s;
    box-sizing: border-box;
  }

  #chat_input:focus { border-color: #228be6; }
  #chat_input:disabled { background: #f8f9fa; color: #868e96; }

  .send-btn {
    width: 42px; height: 42px;
    background: #228be6;
    border: none;
    border-radius: 8px;
    color: white;
    font-size: 16px;
    cursor: pointer;
    display: flex; align-items: center; justify-content: center;
    transition: background .15s;
    flex-shrink: 0;
  }

  .send-btn:hover { background: #1971c2; }
  .send-btn:disabled { background: #adb5bd; cursor: not-allowed; }

  /* ---- Empty state ---- */
  .empty-state {
    text-align: center;
    color: #adb5bd;
    padding: 40px 20px;
    font-size: 14px;
    line-height: 1.8;
  }

  .empty-state-icon { font-size: 36px; margin-bottom: 10px; }
  "
}

# ---------- Inline JavaScript bridge ----------------------------------------

#' Escape a character string for safe embedding in a JS string literal
#'
#' @param x Character vector.
#' @return Character vector with escapes applied.
#' @keywords internal
.js_esc <- function(x) {
  x <- gsub("\\\\", "\\\\\\\\", x)   # backslash
  x <- gsub('"',     '\\"',     x)   # double quote
  x <- gsub("\n",    "\\n",     x)   # newline
  x <- gsub("\r",    "\\r",     x)   # carriage return
  x <- gsub("\t",    "\\t",     x)   # tab
  x
}

build_inline_js <- function(session) {
  # JS is now in inst/www/chat_panel.js and loaded via includeScript()
  # This function is kept for backward compatibility but no longer used.
  NULL
}

# ---------- Helpers (Shiny UI) -------------------------------------------------

# Custom titleBarButton
.titleBarButton <- function(inputId, label, primary = FALSE,
                             icon = NULL, title = NULL) {
  btn_class <- if (isTRUE(primary)) "btn-primary" else "btn-default"
  attrs <- list(
    id    = inputId,
    class = paste("btn btn-sm action-button", btn_class)
  )
  if (!is.null(title)) attrs$title <- title
  label_content <- if (!is.null(icon)) htmltools::tagList(icon, label) else label
  do.call(htmltools::tags$button, c(attrs, list(label_content)))
}

# ---------- Shiny UI ----------------------------------------------------------

chat_title_bar <- function(title = "LLMcoder Chat Panel") {
  htmltools::tags$div(
    style = "display: flex; align-items: center; justify-content: space-between; padding: 8px 16px; background: #2c3e50; color: white; font-size: 15px; font-weight: 600; flex-shrink: 0;",
    htmltools::tags$span(title),
    htmltools::tags$div(
      style = "display: flex; gap: 8px;",
      htmltools::tags$button(
        id = "btn_done",
        class = "btn btn-sm btn-primary",
        style = "background: #228be6; border: none; color: white; padding: 4px 12px; border-radius: 4px; cursor: pointer;",
        "Done"
      )
    )
  )
}

chat_ui <- function(system_prompt_override) {

  tags <- htmltools::tags
  HTML <- htmltools::HTML

  js_src <- system.file("www/chat_panel.js", package = "llmcoder")
  js_content <- readLines(js_src, warn = FALSE)
  js_content <- paste(js_content, collapse = "\n")

  shiny::fillPage(
    tags$head(
      tags$style(chat_css()),
      tags$script(HTML(js_content))
    ),

    # Title bar
    chat_title_bar(),

    # Main layout: sidebar + chat area
    tags$div(class = "chat-layout",

      # ---- Sidebar (fixed width, left) ----
      tags$div(class = "chat-sidebar",

        tags$div(class = "sidebar-logo",
          tags$i(class = "fa fa-comment-dots", style = "margin-right:6px"),
          "LLMcoder"),

        # Context toggle + file browser
        tags$div(class = "sidebar-section",
          tags$div(class = "sidebar-label", "Context"),

          # Toggle for auto session context
          tags$div(class = "toggle-row",
            tags$span("Session Context"),
            tags$label(class = "toggle-switch",
              tags$input(type = "checkbox", id = "toggle_ctx", checked = NA),
              tags$span(class = "toggle-slider")
            )
          ),

          # File browser for manual context
          tags$div(style = "margin-top: 8px",
            tags$button(id = "btn_browse", class = "action-button sidebar-btn",
              tags$i(class = "fa fa-folder-open", style = "width:16px"),
              "Add File to Context"),
            shiny::uiOutput("context_files_ui")
          )
        ),

        # Style selector
        tags$div(class = "sidebar-section",
          tags$div(class = "sidebar-label", "Prompt Style"),
          tags$select(id = "sel_style", class = "sidebar-btn",
            tags$option(value = "general",   "General Assistant"),
            tags$option(value = "code",     "R Code Helper"),
            tags$option(value = "stats",    "Statistics Advisor"),
            tags$option(value = "research", "Research (Psycho)")
          )
        ),

        tags$div(style = "flex: 1"),

        # Utility buttons
        tags$div(class = "sidebar-section",
          tags$button(id = "btn_clear", class = "action-button sidebar-btn",
            tags$i(class = "fa fa-trash-alt", style = "width:16px"),
            "Clear chat"),
          tags$button(id = "btn_export", class = "action-button sidebar-btn",
            tags$i(class = "fa fa-download", style = "width:16px"),
            "Export transcript")
        )
      ),

      # ---- Main chat area (flex: 1) ----
      tags$div(class = "chat-main",

        # Scrollable message area
        tags$div(class = "msg-scroll-area",
          tags$div(id = "msg_container", class = "msg-container",
            tags$div(id = "empty_state", class = "empty-state",
              tags$div(class = "empty-state-icon", HTML("&#128100;")),
              tags$div(tags$strong("LLMcoder Chat Panel")),
              tags$div("Ask me anything about R, data analysis, ",
                tags$br(), "statistics, or your code.")
            )
          ),

          tags$div(id = "thinking_wrap", class = "thinking-wrap",
            style = "display: none",
            tags$div(class = "thinking-dots",
              tags$div(class = "dot"),
              tags$div(class = "dot"),
              tags$div(class = "dot")
            )
          )
        ),

        # Fixed input bar at bottom
        tags$div(class = "chat-input-wrap",
          tags$textarea(
            id          = "chat_input",
            placeholder = "Type a message... (Ctrl+Enter to send)",
            rows        = 1,
            autofocus   = "autofocus"
          ),
          tags$button(id = "btn_send", class = "send-btn", "Send")
        )
      )
    )
  )
}


# ---------- Server logic ------------------------------------------------------

chat_server <- function(input, output, session,
                       system_prompt_override) {

  rv <- shiny::reactiveValues(
    history       = list(),
    thinking      = FALSE,
    session_ctx   = character(0),
    style         = "general",
    context_files = list()   # stores selected files for context
  )

  build_chat_system_prompt <- function(style, ctx_enabled) {
    base <- switch(style,
      general  = "You are a helpful AI assistant for an R user. Answer clearly, accurately, and concisely. Format code with triple backticks marked `r`.",
      code     = "You are an expert R programmer. Write clean, idiomatic R code using tidyverse where appropriate. Show code in ```r fences and explain briefly.",
      stats    = "You are a statistics and R advisor for a psycholinguistics researcher. Explain concepts clearly, suggest appropriate tests, and provide R code for analysis.",
      research = "You are a research assistant for a psycholinguistics PhD student using R. Help with experimental design, mixed-effects models (lme4/lmerTest), data visualisation (ggplot2), and statistical reporting."
    )

    # Append session context if enabled
    if (ctx_enabled && !is.null(system_prompt_override)) {
      base <- paste0(base, "\n\n[SESSION CONTEXT]\n", system_prompt_override)
    } else if (ctx_enabled && length(rv$session_ctx) > 0) {
      base <- paste0(base, "\n\n[SESSION CONTEXT]\n", rv$session_ctx)
    }

    # Append selected file contents
    if (length(rv$context_files) > 0) {
      file_blocks <- vapply(names(rv$context_files), function(fpath) {
        f <- rv$context_files[[fpath]]
        paste0("[FILE: ", f$name, "]\n```\n", f$content, "\n```")
      }, character(1))
      base <- paste0(base, "\n\n[SELECTED FILES]\n", paste(file_blocks, collapse = "\n\n"))
    }

    base
  }

  shiny::observe({
    shiny::req(input$toggle_ctx)
    if (isTRUE(input$toggle_ctx)) {
      tryCatch({
        rv$session_ctx <- session_context_prompt()
      }, error = function(e) {
        rv$session_ctx <- character(0)
      })
    } else {
      rv$session_ctx <- character(0)
    }
  })

  shiny::observeEvent(input$sel_style, {
    rv$style <- input$sel_style
  })

  shiny::observeEvent(input$btn_clear, {
    rv$history <- list()
    session$sendCustomMessage("clearDone", list())
  })

  shiny::observeEvent(input$btn_export, {
    if (length(rv$history) == 0) return()
    lines <- vapply(rv$history, function(m) {
      paste0(toupper(m[["role"]]), ": ", m[["content"]])
    }, character(1))
    txt <- paste(c("# LLMcoder Chat Transcript", Sys.time(), "", lines), collapse = "\n")
    default_path <- file.path(path.expand("~"),
                              paste0("llmcoder_transcript_",
                                     format(Sys.time(), "%Y%m%d_%H%M%S"), ".txt"))
    path <- tryCatch({
      rstudioapi::selectFile(label = "Save transcript as", path = default_path,
                             filter = "Text files (*.txt)")
    }, error = function(e) default_path)
    if (!is.null(path) && nchar(path) > 0) {
      writeLines(txt, path)
      rstudioapi::showDialog("Export", paste("Transcript saved to:\n", path))
    }
  })


  # ---- Browse and add file to context ----
  shiny::observeEvent(input$btn_browse, {
    path <- tryCatch({
      rstudioapi::selectFile(
        label = "Select file to add to context",
        filter = "All files (*.*)"
      )
    }, error = function(e) NULL)

    if (is.null(path) || !file.exists(path)) return()

    # Avoid duplicates
    if (path %in% names(rv$context_files)) return()

    # Read file content (try UTF-8, fallback to Latin-1)
    content <- tryCatch({
      readLines(path, encoding = "UTF-8", warn = FALSE)
    }, error = function(e) {
      tryCatch({
        readLines(path, encoding = "latin1", warn = FALSE)
      }, error = function(e2) character(0))
    })

    if (length(content) > 0) {
      rv$context_files[[path]] <- list(
        name    = basename(path),
        path    = path,
        content = paste(content, collapse = "\n")
      )
    } else {
      rstudioapi::showDialog("LLMcoder",
        paste("Could not read file:\n", path))
    }
  })


  # ---- Remove file from context ----
  shiny::observeEvent(input$remove_file, {
    req(input$remove_file)
    fname <- input$remove_file
    rv$context_files[[fname]] <- NULL
  })


  # ---- Render selected files list in sidebar ----
  output$context_files_ui <- shiny::renderUI({
    files <- rv$context_files
    if (length(files) == 0) {
      return(tags$div(style = "font-size: 11px; color: #868e96; padding: 4px 0;",
                      "No files added"))
    }

    tag_list <- lapply(names(files), function(fpath) {
      f <- files[[fpath]]
      tags$div(
        style = "display: flex; align-items: center; justify-content: space-between;
                padding: 4px 6px; margin-bottom: 3px;
                background: #f8f9fa; border-radius: 4px; font-size: 12px;",
        tags$span(
          style = "overflow: hidden; text-overflow: ellipsis; white-space: nowrap; max-width: 140px;",
          tags$i(class = "fa fa-file-code", style = "margin-right: 4px; color: #495057"),
          f$name
        ),
        tags$button(
          type  = "button",
          class = "btn-remove-file",
          onclick = sprintf("Shiny.setInputValue('remove_file', '%s')", fpath),
          style  = "background: none; border: none; color: #e74c3c; cursor: pointer; font-size: 14px; padding: 0 4px;",
          "x"
        )
      )
    })
    do.call(tags$div, tag_list)
  })

  # Make context_files_ui available to UI
  shiny::outputOptions(output, "context_files_ui", suspendWhenHidden = FALSE)


  shiny::observeEvent(input$doChat, {
    text <- input$doChat
    if (!nzchar(trimws(text))) return()

    ctx_enabled <- isTRUE(input$toggle_ctx)
    sys_prompt  <- build_chat_system_prompt(rv$style, ctx_enabled)

    rv$history  <- c(rv$history, list(list(role = "user", content = text)))
    rv$thinking <- TRUE

    msgs <- c(list(list(role = "system", content = sys_prompt)), rv$history)

    resp <- tryCatch({
      message("[llmcoder] Calling LLM with text: ", substr(text, 1, 80))
      safe_call_llm_history(msgs)
    }, error = function(e) {
      message("[llmcoder] LLM error: ", conditionMessage(e))
      list(error = conditionMessage(e))
    })

    rv$thinking <- FALSE
    session$sendCustomMessage("thinkingDone", list())

    if (is.list(resp) && !is.null(resp$error)) {
      session$sendCustomMessage("appendMsg", list(
        role = "error",
        html = paste0("<p>Error: ", escape_html(resp$error), "</p>")
      ))
      session$sendCustomMessage("inputEnabled", list(enabled = TRUE))
      return()
    }

    rv$history <- c(rv$history, list(list(role = "assistant", content = resp)))
    message("[llmcoder] resp type: ", typeof(resp),
            " is_list:", is.list(resp),
            " is_fn:", is.function(resp))
    html <- render_markdown_html(resp)

    session$sendCustomMessage("appendMsg", list(
      role  = "assistant",
      html  = html
    ))
    session$sendCustomMessage("inputEnabled", list(enabled = TRUE))
  })

  # ---- Run code in console ----
  shiny::observeEvent(input$doRunCode, {
    code <- input$doRunCode
    if (!nzchar(trimws(code))) return()
    tryCatch({
      rstudioapi::sendToConsole(code, execute = TRUE, focus = FALSE)
    }, error = function(e) {
      rstudioapi::showDialog("LLMcoder", paste("Could not send to console:", e$message))
    })
  })

  # ---- Insert code into script ----
  shiny::observeEvent(input$doInsertCode, {
    code <- input$doInsertCode
    if (!nzchar(trimws(code))) return()
    tryCatch({

      rstudioapi::insertText(code)
    }, error = function(e) {

      tryCatch({
        rstudioapi::sendToConsole(code, execute = FALSE, focus = FALSE)
        rstudioapi::showDialog("LLMcoder", "No active document. Code sent to console (not executed).")
      }, error = function(e2) {
        rstudioapi::showDialog("LLMcoder", paste("Could not insert code:", e$message))
      })
    })
  })

  shiny::observeEvent(input$btn_done, {
    shiny::stopApp()
  })
}


# ---------- Markdown renderer -------------------------------------------------

# function triggers "cannot coerce type 'closure'" in R's C internals.
# All markdown rendering uses stringi (ICU regex) with manual replacement logic,
# because stri_replace_all_regex() does NOT support function replacement.

escape_html <- function(s) {
  if (!is.character(s) || length(s) == 0) return("")


  s <- gsub("&", "&amp;",  s, fixed = TRUE)
  s <- gsub("<", "&lt;",   s, fixed = TRUE)
  s <- gsub(">", "&gt;",   s, fixed = TRUE)
  s
}

# Workaround: stri_replace_all() doesn't support function replacement.
# Use stri_match_all + manual string reconstruction (right-to-left).
# New stringi API: pass regex options via ... to regex(pattern, ...)
replace_all_regex_fn <- function(str_vec, pattern, repl_fn, ...) {
  vapply(seq_along(str_vec), function(i) {
    s <- str_vec[i]
    matches <- stri_match_all(s, ..., regex = pattern)[[1]]
    if (is.na(matches[1, 1])) return(s)
    n <- nrow(matches)
    positions <- stri_locate_all(s, ..., regex = pattern)[[1]]
    result <- ""
    prev_end <- nchar(s) + 1L
    for (j in rev(seq_len(n))) {
      m_start <- positions[j, 1]
      m_end   <- positions[j, 2]
      repl <- repl_fn(c(matches[j, 1], matches[j, 2:ncol(matches)]))
      result <- paste0(
        if (m_end + 1L <= prev_end - 1L)
          stri_sub(s, m_end + 1L, prev_end - 1L)
        else "",
        repl,
        result
      )
      prev_end <- m_start
    }
    paste0(stri_sub(s, 1L, prev_end - 1L), result)
  }, character(1), USE.NAMES = FALSE)
}

render_markdown_html <- function(text) {
  if (!is.character(text) || length(text) == 0) {
    text <- if (is.function(text)) {
      "[LLM returned a function object -- possible API misconfiguration]"
    } else {
      as.character(text)
    }
  }
  rendered <- as.character(text)[1]

  # ---- Code fences ----

  lines <- unlist(strsplit(rendered, "\n"))
  in_code_block <- FALSE
  code_lang <- ""
  code_lines <- character(0)
  output_parts <- character(0)

  for (i in seq_along(lines)) {
    line <- lines[i]


    if (grepl("^```[A-Za-z0-9_]*$", line)) {
      if (!in_code_block) {

        in_code_block <- TRUE
        code_lang <- gsub("`", "", line)
        if (code_lang == "") code_lang <- "text"
        next
      } else {

        in_code_block <- FALSE
        inner <- paste(code_lines, collapse = "\n")
        code_lines <- character(0)


        escaped <- escape_html(inner)
        label <- if (code_lang == "r" || code_lang == "R") "R" else toupper(code_lang)

        if (code_lang == "r" || code_lang == "R") {
          run_btn <- paste0("<button class='btn-run-code' data-code='",
                           URLencode(inner, reserved = TRUE), "'>Run</button>")
          insert_btn <- paste0("<button class='btn-insert-code' data-code='",
                               URLencode(inner, reserved = TRUE), "'>Insert</button>")
          buttons <- paste0(run_btn, " ", insert_btn)
        } else {
          buttons <- ""
        }

        pre_block <- paste0("<pre><div class='code-header'><span class='code-lang'>",
                            label, "</span>", buttons,
                            "</div><code>", escaped, "</code></pre>")
        output_parts <- c(output_parts, pre_block)
        next
      }
    }

    if (in_code_block) {
      code_lines <- c(code_lines, line)
    } else {
      output_parts <- c(output_parts, line)
    }
  }


  rendered <- paste(output_parts, collapse = "\n")

  # ---- Process Markdown inline formatting ----

  blocks <- unlist(strsplit(rendered, "\n\n+"))
  blocks <- blocks[nchar(gsub("\\s", "", blocks)) > 0]

  render_block <- function(block) {

    if (grepl("^<pre>", block)) {
      return(block)
    }

    if (grepl("^### ", block, perl = TRUE)) {
      title <- substring(block, 5)
      return(paste0("<h4>", title, "</h4>"))
    }
    if (grepl("^## ", block, perl = TRUE)) {
      title <- substring(block, 4)
      return(paste0("<h3>", title, "</h3>"))
    }
    if (grepl("^# ", block, perl = TRUE)) {
      title <- substring(block, 3)
      return(paste0("<h2>", title, "</h2>"))
    }

    if (grepl("^[-*] ", block, perl = TRUE)) {
      items <- unlist(strsplit(block, "\n"))
      items <- sapply(items, function(item) {
        content <- sub("^[-*] ", "", item, perl = TRUE)
        paste0("<li>", content, "</li>")
      })
      return(paste0("<ul>\n", paste0(items, collapse = "\n"), "\n</ul>"))
    }

    block <- escape_html(block)

    m <- gregexpr("`[^`]+`", block, perl = TRUE)
    if (m[[1]][1] != -1) {
      matches <- regmatches(block, m)[[1]]
      for (j in rev(seq_along(matches))) {
        code <- substring(matches[j], 2, nchar(matches[j]) - 1)
        pos <- m[[1]][j]
        len <- attr(m[[1]], "match.length")[j]
        block <- paste0(
          substr(block, 1, pos - 1),
          paste0("<code>", code, "</code>"),
          substr(block, pos + len, nchar(block))
        )
      }
    }

    m <- gregexpr("\\*\\*([^*]+)\\*\\*", block, perl = TRUE)
    if (m[[1]][1] != -1) {
      matches <- regmatches(block, m)[[1]]
      for (j in rev(seq_along(matches))) {
        content <- gsub("\\*\\*", "", matches[j], fixed = TRUE)
        pos <- m[[1]][j]
        len <- attr(m[[1]], "match.length")[j]
        block <- paste0(
          substr(block, 1, pos - 1),
          paste0("<strong>", content, "</strong>"),
          substr(block, pos + len, nchar(block))
        )
      }
    }

    m <- gregexpr("(?<![*])\\*([^*]+)\\*(?![*])", block, perl = TRUE)
    if (m[[1]][1] != -1) {
      matches <- regmatches(block, m)[[1]]
      for (j in rev(seq_along(matches))) {
        content <- gsub("\\*", "", matches[j], fixed = TRUE)
        pos <- m[[1]][j]
        len <- attr(m[[1]], "match.length")[j]
        block <- paste0(
          substr(block, 1, pos - 1),
          paste0("<em>", content, "</em>"),
          substr(block, pos + len, nchar(block))
        )
      }
    }
    paste0("<p>", block, "</p>")
  }

  rendered <- paste(sapply(blocks, render_block), collapse = "\n")

  htmltools::HTML(rendered)
}



# ---------- JS <-> R bridge handlers -----------------------------------------

chat_js_handlers <- function(session) {
  invisible(NULL)
}


# ---------- Public entry point -------------------------------------------------

#' @export
addin_chat_panel <- function(system_prompt_override = NULL) {
  if (!requireNamespace("rstudioapi", quietly = TRUE) ||
      !rstudioapi::isAvailable()) {
    stop("addin_chat_panel() must be run from within RStudio.", call. = FALSE)
  }
  if (!requireNamespace("jsonlite", quietly = TRUE)) {
    stop("The 'jsonlite' package is required for addin_chat_panel(). ",
         "Install it with: install.packages('jsonlite')", call. = FALSE)
  }
  chat_gadget(system_prompt_override = system_prompt_override)
}


# ---------- Gadget factory ----------------------------------------------------

chat_gadget <- function(system_prompt_override = NULL) {
  ui <- chat_ui(system_prompt_override)
  server <- function(input, output, session) {
    chat_server(input, output, session, system_prompt_override)
  }
  viewer <- shiny::dialogViewer(
    "LLMcoder Chat Panel",
    width  = 1400,
    height = 950
  )
  shiny::runGadget(ui, server, viewer = viewer)
}


# ---------- LLM multi-turn API -----------------------------------------------

safe_call_llm_history <- function(messages, system_prompt_override = NULL) {
  tryCatch(
    call_llm_history(messages, system_prompt_override),
    error = function(e) list(error = conditionMessage(e))
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/editor.R
\name{notify}
\alias{notify}
\title{Emit a status message to the R console}
\usage{
notify(msg)
}
\arguments{
\item{msg}{Character.  The message text.}
}
\value{
Invisible \code{NULL}.
}
\description{
Prefixes the message with \verb{[llmcoder] } so users can distinguish addin
output from their own code output.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api.R
\name{call_llm}
\alias{call_llm}
\title{Call the configured LLM}
\usage{
call_llm(prompt, system_prompt, context = NULL)
}
\arguments{
\item{prompt}{Character. The user-facing instruction.}

\item{system_prompt}{Character. The system-level instruction for the model.}

\item{context}{Character or \code{NULL}. Surrounding R code sent as
additional context (prepended to \code{prompt}).}
}
\value{
Character string containing the model's response text.
}
\description{
Unified dispatch function that reads provider, model, and credentials from
options (set via \code{\link[=llmcoder_setup]{llmcoder_setup()}} or \strong{Addins \→ LLMcoder Settings}) and
forwards the request to the appropriate backend.
}
\details{
Supported providers:
\describe{
\item{\code{"openai"}}{OpenAI Chat Completions API
(\verb{https://api.openai.com/v1}).}
\item{\code{"anthropic"}}{Anthropic Messages API
(\verb{https://api.anthropic.com/v1/messages}).}
\item{\code{"deepseek"}}{DeepSeek Chat API, OpenAI-compatible
(\verb{https://api.deepseek.com/v1}).}
\item{\code{"ollama"}}{Local Ollama server (default
\verb{http://localhost:11434}).  No API key required.}
\item{\code{"groq"}}{Groq Cloud API, OpenAI-compatible
(\verb{https://api.groq.com/openai/v1}).  Extremely fast inference.}
\item{\code{"together"}}{Together AI API, OpenAI-compatible
(\verb{https://api.together.xyz/v1}).  Wide open-source model selection.}
\item{\code{"openrouter"}}{OpenRouter API, OpenAI-compatible
(\verb{https://openrouter.ai/api/v1}).  Unified gateway to 100+ models.}
\item{\code{"custom"}}{Any OpenAI-compatible server.  Set
\code{llmcoder.custom_url} to the base URL (e.g.\
\code{"http://localhost:1234/v1"} for LM Studio).}
}
}
\examples{
\dontrun{
llmcoder_setup("ollama", model = "llama3")
resp <- call_llm(
  prompt        = "Write R code to compute the mean of a numeric vector",
  system_prompt = "You are an R programming assistant.",
  context       = NULL
)
cat(resp)
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/setup.R
\name{llmcoder_setup}
\alias{llmcoder_setup}
\title{Configure LLMcoder for the current session}
\usage{
llmcoder_setup(
  provider = c("openai", "anthropic", "deepseek", "ollama", "groq", "together",
    "openrouter", "custom"),
  api_key = NULL,
  model = NULL,
  context_lines = 40L,
  ollama_url = "http://localhost:11434",
  custom_url = ""
)
}
\arguments{
\item{provider}{Character.  One of \code{"openai"}, \code{"anthropic"},
\code{"deepseek"}, \code{"ollama"}, \code{"groq"}, \code{"together"}, \code{"openrouter"}, or
\code{"custom"}.}

\item{api_key}{Character.  Your API key.  Not required when
\code{provider = "ollama"}.}

\item{model}{Character.  Model identifier.  If \code{NULL} or \code{""}, a sensible
default is chosen for the provider (see \strong{Details}).}

\item{context_lines}{Integer.  Number of lines of code above the cursor that
are sent as context to the LLM (default \code{40}).  Higher values improve
suggestion quality but increase latency and token cost.}

\item{ollama_url}{Character.  Base URL of the Ollama server (default
\code{"http://localhost:11434"}).  Only used when \code{provider = "ollama"}.}

\item{custom_url}{Character.  Base URL of a custom OpenAI-compatible server
(e.g. \code{"http://localhost:1234/v1"} for LM Studio).  Only used when
\code{provider = "custom"}.}
}
\value{
Invisible \code{NULL}.
}
\description{
Sets the LLM provider, API key, model, and related options for the current R
session.  For \strong{permanent} configuration that survives restarts, use
\strong{Addins > LLMcoder Settings}, which writes to \code{~/.Rprofile}.
}
\details{
\strong{Provider defaults:}\tabular{lll}{
   Provider \tab Default model \tab Notes \cr
   \code{openai} \tab \verb{gpt-4o-mini} \tab Fast, cost-effective \cr
   \code{anthropic} \tab \code{claude-sonnet-4-20250514} \tab Strongest reasoning \cr
   \code{deepseek} \tab \code{deepseek-chat} \tab Very cheap, great code quality \cr
   \code{ollama} \tab \code{llama3} \tab No API key, fully local \cr
   \code{groq} \tab \verb{llama-3.3-70b-versatile} \tab Extremely fast inference \cr
   \code{together} \tab \verb{meta-llama/Llama-3-70b-chat-hf} \tab Large open-source model choice \cr
   \code{openrouter} \tab \verb{openai/gpt-4o-mini} \tab Unified gateway for 100+ models \cr
   \code{custom} \tab \code{""} (must specify) \tab Any OpenAI-compat endpoint \cr
}
}
\examples{
\dontrun{
# OpenAI
llmcoder_setup("openai", api_key = Sys.getenv("OPENAI_API_KEY"))
llmcoder_setup("openai", api_key = Sys.getenv("OPENAI_API_KEY"), model = "gpt-4o")

# Anthropic Claude
llmcoder_setup("anthropic", api_key = Sys.getenv("ANTHROPIC_API_KEY"))

# DeepSeek (cheapest, excellent code quality)
llmcoder_setup("deepseek", api_key = Sys.getenv("DEEPSEEK_API_KEY"))

# Ollama — fully local, no API key needed
llmcoder_setup("ollama", model = "qwen2.5-coder:7b")
llmcoder_setup("ollama", model = "codellama:13b",
               ollama_url = "http://192.168.1.10:11434")  # remote server

# Groq — extremely fast inference on open models
llmcoder_setup("groq",
  api_key = Sys.getenv("GROQ_API_KEY"),
  model   = "llama-3.3-70b-versatile")

# Together AI — wide open-source model selection
llmcoder_setup("together",
  api_key = Sys.getenv("TOGETHER_API_KEY"),
  model   = "mistralai/Mixtral-8x7B-Instruct-v0.1")

# OpenRouter — unified gateway, supports 100+ models
llmcoder_setup("openrouter",
  api_key = Sys.getenv("OPENROUTER_API_KEY"),
  model   = "anthropic/claude-3.5-sonnet")

# LM Studio or any OpenAI-compatible local server
llmcoder_setup("custom",
  api_key    = "lm-studio",
  model      = "local-model",
  custom_url = "http://localhost:1234/v1")

# Reduce context window to save tokens
llmcoder_setup("openai",
  api_key       = Sys.getenv("OPENAI_API_KEY"),
  context_lines = 20L)
}

}
\seealso{
\code{\link[=llmcoder_config]{llmcoder_config()}}, \code{\link[=addin_settings]{addin_settings()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api.R
\name{call_ollama}
\alias{call_ollama}
\title{Ollama local API (uses the OpenAI-compatible /v1 endpoint, Ollama >= 0.1.24)}
\usage{
call_ollama(prompt, system_prompt, model, base_url)
}
\description{
No API key is required.  The Ollama server must be running locally;
start it with \verb{ollama serve} in a terminal.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/editor.R
\name{extract_comment_at_cursor}
\alias{extract_comment_at_cursor}
\title{Extract the comment text at the current cursor position}
\usage{
extract_comment_at_cursor(ctx)
}
\arguments{
\item{ctx}{An \code{rstudio_editor_context} object from
\code{\link[rstudioapi:rstudio-editors]{rstudioapi::getSourceEditorContext()}}.}
}
\value{
A named list with four components:
\describe{
\item{\code{comment}}{Character.  The comment text with the leading \verb{#}
character(s) and optional space stripped.}
\item{\code{row}}{Integer.  1-based row index of the comment line in the
document.}
\item{\code{full_line}}{Character.  The raw full-line text as it appears in
the editor.}
\item{\code{indent}}{Character.  The leading whitespace of the line
(used to preserve indentation when inserting generated code).}
}
}
\description{
Reads the line at the cursor position from the active editor context and
returns its components.  Throws an informative error if the cursor is not
positioned on a comment line (i.e., a line starting with \verb{#}, possibly
preceded by whitespace).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/addins.R
\name{addin_generate_from_comment}
\alias{addin_generate_from_comment}
\title{Generate R code from a comment (silent insert)}
\usage{
addin_generate_from_comment()
}
\value{
Invisible \code{NULL} (called for side-effects).
}
\description{
Places the cursor on a line beginning with \verb{#}, then triggers this addin
(default shortcut: \code{Ctrl+Shift+G} on Windows/Linux, \code{Cmd+Shift+G} on macOS).
The LLM reads the comment text and the surrounding code context, then inserts
the generated R code on the line immediately below the comment.
}
\details{
The addin extracts the text of the comment at the cursor position and up to
\code{getOption("llmcoder.context_lines", 40L)} lines of preceding code as
context.  The provider, model, and API key are taken from options set by
\code{\link[=llmcoder_setup]{llmcoder_setup()}} or the \strong{LLMcoder Settings} addin.

No dialog is shown; code is inserted immediately.  Use
\code{\link[=addin_generate_with_preview]{addin_generate_with_preview()}} if you prefer to review the output first.
}
\seealso{
\code{\link[=addin_generate_with_preview]{addin_generate_with_preview()}}, \code{\link[=llmcoder_setup]{llmcoder_setup()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/addins.R
\name{safe_get_ctx}
\alias{safe_get_ctx}
\title{Safely obtain the active editor context}
\usage{
safe_get_ctx()
}
\description{
Safely obtain the active editor context
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gadgets.R
\name{write_rprofile}
\alias{write_rprofile}
\title{Write llmcoder options to ~/.Rprofile}
\usage{
write_rprofile(provider, model, api_key, ctx_lines, ollama_url, custom_url)
}
\arguments{
\item{provider}{Character.  Provider identifier (see \code{\link[=llmcoder_setup]{llmcoder_setup()}}).}

\item{model}{Character.  Model name.}

\item{api_key}{Character.  API key (may be \code{""} for Ollama).}

\item{ctx_lines}{Integer.    Number of context lines.}

\item{ollama_url}{Character.  Ollama base URL.}

\item{custom_url}{Character.  Custom endpoint base URL.}
}
\value{
Invisible \code{NULL}.  Called for its side-effect of writing to
\verb{~/.Rprofile}.
}
\description{
Writes (or replaces) an \verb{# --- llmcoder ---} block in the user's
\verb{~/.Rprofile} so that llmcoder settings persist across R sessions.
}
\examples{
\dontrun{
write_rprofile(
  provider   = "ollama",
  model      = "llama3",
  api_key    = "",
  ctx_lines  = 40L,
  ollama_url = "http://localhost:11434",
  custom_url = ""
)
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api.R
\name{build_explain_prompt}
\alias{build_explain_prompt}
\title{System prompt for code explanation}
\usage{
build_explain_prompt()
}
\value{
Character string: the system prompt sent to the LLM for the
explain workflow.
}
\description{
Returns the system prompt instructing the LLM to write R comments
explaining the user's selected code.
}
\examples{
\dontrun{
build_explain_prompt()
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/addins.R
\name{addin_generate_with_preview}
\alias{addin_generate_with_preview}
\title{Generate R code with an editable preview dialog}
\usage{
addin_generate_with_preview()
}
\value{
Invisible \code{NULL} (called for side-effects).
}
\description{
Same as \code{\link[=addin_generate_from_comment]{addin_generate_from_comment()}} but opens a Shiny gadget so you can
review and optionally edit the generated code before it is inserted into the
editor.  Recommended shortcut: \code{Ctrl+Shift+P} / \code{Cmd+Shift+P}.
}
\details{
The preview dialog shows the generated code in an editable text area.
Click \strong{Insert} to place it in the editor, or close the dialog to discard
the result.
}
\seealso{
\code{\link[=addin_generate_from_comment]{addin_generate_from_comment()}}, \code{\link[=llmcoder_setup]{llmcoder_setup()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/addins.R
\name{.collect_last_error}
\alias{.collect_last_error}
\title{Collect the most recent R error message using multiple strategies}
\usage{
.collect_last_error()
}
\description{
Tries, in order: rlang::last_error(), .Last.error (base R),
.Last.error condition message.
Returns NULL if nothing is found.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/editor.R
\name{gather_context}
\alias{gather_context}
\title{Collect N lines of surrounding code above the cursor}
\usage{
gather_context(ctx, row, n = 30)
}
\arguments{
\item{ctx}{An \code{rstudio_editor_context} object.}

\item{row}{Integer.  1-based row of the comment line (context is taken from
rows \code{max(1, row - n)} to \code{row - 1}).}

\item{n}{Integer.  Maximum number of context lines (default \code{30}).}
}
\value{
A single character string (may be \code{""} if \code{row == 1}).
}
\description{
Returns a character string containing the \code{n} lines of source code
immediately above the comment line, joined by newlines.  This is sent to the
LLM as context so that it can infer variable names, existing code style, and
already-loaded packages.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/addins.R
\name{safe_call_llm}
\alias{safe_call_llm}
\title{Safely call the LLM, catching API errors}
\usage{
safe_call_llm(prompt, system_prompt, context)
}
\description{
Safely call the LLM, catching API errors
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api.R
\name{call_llm_history}
\alias{call_llm_history}
\title{Call the configured LLM with full message history}
\usage{
call_llm_history(messages, system_prompt_override = NULL)
}
\arguments{
\item{messages}{List of messages, each element being
\code{list(role = "system"|"user"|"assistant", content = "...")}.
The \strong{last} message must have \code{role = "user"}.}

\item{system_prompt_override}{Optional character string.  If supplied,
it \strong{replaces} the system role message in \code{messages} entirely
(useful for swapping in a custom persona at call-time).}
}
\value{
Character string: the model's response text.
}
\description{
Like \code{\link[=call_llm]{call_llm()}} but accepts a \code{messages} list that preserves the full
conversation context across multiple turns.  This is the engine behind
\code{\link[=addin_chat_panel]{addin_chat_panel()}}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/addins.R
\name{addin_fix_selected_error}
\alias{addin_fix_selected_error}
\title{Fix an error by selecting its text}
\usage{
addin_fix_selected_error()
}
\value{
Invisible \code{NULL} (called for side-effects).
}
\description{
Select the error message text in the editor (or paste it into a temporary
comment), then trigger this addin.  The addin pairs the selected text with
the complete source file currently open in the editor and asks the LLM for a
fix, displaying the result in a review dialog.
}
\details{
This addin is the recommended fallback when \code{\link[=addin_fix_console_error]{addin_fix_console_error()}} does
not detect an error automatically (e.g., because the error occurred inside a
\code{tryCatch()} block or in a separate R process).

\strong{Workflow}
\enumerate{
\item Copy the error message from the console.
\item Paste it anywhere in the source file, or simply select it in the console
output if your terminal supports that.
\item Select the error text in the editor.
\item Trigger this addin.
\item Review and apply the suggested fix.
}
}
\seealso{
\code{\link[=addin_fix_console_error]{addin_fix_console_error()}}, \code{\link[=llmcoder_setup]{llmcoder_setup()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api.R
\name{build_fix_prompt}
\alias{build_fix_prompt}
\title{System prompt for error fixing}
\usage{
build_fix_prompt()
}
\value{
Character string: the system prompt sent to the LLM for the
fix workflow.
}
\description{
Returns the system prompt instructing the LLM to diagnose an R error and
produce corrected code.
}
\examples{
\dontrun{
build_fix_prompt()
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gadgets.R
\name{gadget_css}
\alias{gadget_css}
\title{Shared CSS for all gadgets}
\usage{
gadget_css()
}
\description{
Shared CSS for all gadgets
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api.R
\name{default_model}
\alias{default_model}
\title{Default model name per provider}
\usage{
default_model(provider)
}
\arguments{
\item{provider}{Character. Provider identifier (see \code{\link[=call_llm]{call_llm()}}).}
}
\value{
Character string with the default model name.
}
\description{
Returns a sensible default model name when the user has not specified one
explicitly.
}
\examples{
\dontrun{
default_model("openai")
default_model("anthropic")
default_model("ollama")
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chat.R
\name{escape_html}
\alias{escape_html}
\title{Very simple markdown-to-HTML renderer for chat messages}
\usage{
escape_html(s)
}
\arguments{
\item{s}{Character string. Raw markdown text.}
}
\value{
Character string. Sanitised HTML fragment.
}
\description{
Handles fenced code blocks (\verb{```r ... ```}), inline code, bold, italic,
headings, and unordered lists.  No external dependencies required.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/editor.R
\name{insert_after_row}
\alias{insert_after_row}
\title{Insert text immediately after a given row in the editor}
\usage{
insert_after_row(text, row, indent = "")
}
\arguments{
\item{text}{Character.  Code to insert; may contain newlines.}

\item{row}{Integer.  1-based row \emph{after} which the text is inserted.}

\item{indent}{Character.  Leading whitespace prepended to every inserted
line (default \code{""}).}
}
\value{
Invisible \code{NULL} (called for side-effects).
}
\description{
Inserts one or more lines of text at the beginning of the row that follows
\code{row} in the currently active source editor.  Each line is prepended with
\code{indent} to match the indentation level of the originating comment.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api.R
\name{call_ollama_history}
\alias{call_ollama_history}
\title{Ollama multi-turn (OpenAI-compatible endpoint)}
\usage{
call_ollama_history(messages, model, base_url)
}
\description{
Ollama multi-turn (OpenAI-compatible endpoint)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chat.R
\name{chat_js_handlers}
\alias{chat_js_handlers}
\title{Register Shiny custom message handlers for the Chat Panel}
\usage{
chat_js_handlers(session)
}
\arguments{
\item{session}{Shiny session object.}
}
\description{
Registers R-side handlers that respond to JavaScript events dispatched
by the Chat Panel UI (code-run button clicks, etc.).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api.R
\name{ollama_list_models}
\alias{ollama_list_models}
\title{List models available on a running Ollama server}
\usage{
ollama_list_models(
  base_url = getOption("llmcoder.ollama_url", "http://localhost:11434")
)
}
\arguments{
\item{base_url}{Character. Ollama base URL.  Defaults to the value of
\code{getOption("llmcoder.ollama_url", "http://localhost:11434")}.}
}
\value{
Character vector of model tag names, or \code{NULL} if Ollama is not
reachable.
}
\description{
Queries \code{GET /api/tags} on the local Ollama REST API and returns the names
of all installed models.  Useful for populating the model selector in the
Settings gadget.
}
\details{
Ollama must be running (\verb{ollama serve}) before calling this function.
Models are installed with \verb{ollama pull <model>} from the terminal.
}
\examples{
\dontrun{
ollama_list_models()
# [1] "llama3:latest"  "qwen2.5-coder:7b"  "mistral:latest"
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/addins.R
\name{addin_explain_code}
\alias{addin_explain_code}
\title{Explain selected R code as inline comments}
\usage{
addin_explain_code()
}
\value{
Invisible \code{NULL} (called for side-effects).
}
\description{
Select a block of R code in the editor, then trigger this addin
(recommended shortcut: \code{Ctrl+Shift+E} / \code{Cmd+Shift+E}).  An explanation is
inserted as \verb{#} comment lines immediately \strong{above} the selected code block.
}
\details{
The LLM receives the selected code and is instructed to produce a concise,
human-readable explanation — focusing on \emph{what} the code does and \emph{why},
not on basic R syntax.  Every output line is prefixed with \verb{# } so the
explanation is valid R that can be left in the source file.
}
\seealso{
\code{\link[=addin_generate_from_comment]{addin_generate_from_comment()}}, \code{\link[=llmcoder_setup]{llmcoder_setup()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/setup.R
\name{llmcoder_config}
\alias{llmcoder_config}
\title{Show the current LLMcoder configuration}
\usage{
llmcoder_config()
}
\value{
An object of class \code{"llmcoder_config"}: a named list with elements
\code{provider}, \code{model}, \code{api_key}, \code{context_lines}, \code{ollama_url}, and
\code{custom_url}.  The API key is masked for security.  When printed, it
displays in a human-readable table.
}
\description{
Returns (and prints) the active provider, model, API key (masked),
context-lines setting, and any provider-specific URLs.
}
\examples{
# Show current configuration (reads from option values)
llmcoder_config()

# Capture the config as a list for programmatic use
cfg <- llmcoder_config()
cfg$provider
cfg$model

}
\seealso{
\code{\link[=llmcoder_setup]{llmcoder_setup()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/session-context.R
\name{.get_console_history}
\alias{.get_console_history}
\title{Console history from rstudioapi with ~/.Rhistory fallback}
\usage{
.get_console_history(max_hist = 30L)
}
\description{
Console history from rstudioapi with ~/.Rhistory fallback
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/addins.R
\name{addin_fix_console_error}
\alias{addin_fix_console_error}
\title{Fix the last console error automatically}
\usage{
addin_fix_console_error()
}
\value{
Invisible \code{NULL} (called for side-effects).
}
\description{
After running code that produces an error in the R console, trigger this
addin (recommended shortcut: \code{Ctrl+Shift+F} / \code{Cmd+Shift+F}).
}
\details{
The addin attempts to recover the most recent error message using several
strategies, in order of priority:

\enumerate{
\item The \code{rlang} last-error store (\code{rlang::last_error()}), which captures
errors thrown by \pkg{rlang}-aware packages and the tidyverse.
\item Base R's \code{.Last.error} binding (set whenever an unhandled condition
reaches the top level).
\item The \code{.Last.error.trace} character vector written by some versions of
\pkg{rlang}.
}

The complete source file currently open in the editor is also sent to the LLM
as context.  The LLM returns the entire corrected file, with changed lines
annotated as \verb{# FIX: <reason>}.  A diff-style preview dialog lets you review
and edit the fix before applying it.

\strong{Workflow}
\enumerate{
\item Run code — error appears in console.
\item Trigger this addin.
\item Review the fix in the preview dialog → click \strong{Apply Fix}.
}

If no recent error is detected, a dialog explains the possible reasons and
suggests using \code{\link[=addin_fix_selected_error]{addin_fix_selected_error()}} instead.
}
\seealso{
\code{\link[=addin_fix_selected_error]{addin_fix_selected_error()}}, \code{\link[=llmcoder_setup]{llmcoder_setup()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/addins.R
\name{clean_code_output}
\alias{clean_code_output}
\title{Strip markdown code fences from LLM output}
\usage{
clean_code_output(code)
}
\arguments{
\item{code}{Character string returned by an LLM, possibly wrapped in
\verb{ ```r} code fences.}
}
\value{
Character string with fences removed.  If no fences are found, the
input is returned as-is.
}
\description{
Strips common markdown code fences from LLM output so the raw code can be
inserted into the editor.
}
\examples{
\dontrun{
raw <- "\n```r\nx <- mean(1:10)\nprint(x)\n```\n"
clean_code_output(raw)
clean_code_output("no fences here")
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chat.R
\name{addin_chat_panel}
\alias{addin_chat_panel}
\alias{chat_gadget}
\title{Open the LLMcoder Chat Panel}
\usage{
addin_chat_panel(system_prompt_override = NULL)

chat_gadget(system_prompt_override = NULL)
}
\arguments{
\item{system_prompt_override}{Override the auto-generated system prompt.
Use with caution; this replaces the default persona entirely.
Pass \code{NULL} (the default) to use the built-in prompt templates.}
}
\value{
Invisible \code{NULL} (launches a Shiny gadget and does not
return until the gadget is closed).
}
\description{
Launches an interactive multi-turn chat powered by your configured LLM
directly inside RStudio.  The panel supports:
\itemize{
\item Multi-turn conversation history.
\item Session Context awareness (loaded packages, global objects,
source editor contents, console history).
\item One-click code execution: each code block (marked
\verb{```r}) includes a
\strong{Run} button that sends the code to the R console.
\item Prompt style presets: General, R Code Helper, Statistics Advisor,
Research (Psycho).
\item Session context toggle.
\item Transcript export.
}
}
\details{
Bind a keyboard shortcut in RStudio (\strong{Tools -> Modify Keyboard
Shortcuts...}) to \code{addin_chat_panel()} for quick access
(\kbd{Ctrl+Shift+L} is a common choice).
}
\examples{
\dontrun{
# Open the chat panel (assigns Ctrl+Shift+L in RStudio shortcuts)
addin_chat_panel()

# Start with a custom system prompt
addin_chat_panel(system_prompt_override = "You are a strict R reviewer.")
}

}
\seealso{
\code{\link[=llmcoder_setup]{llmcoder_setup()}}, \code{\link[=addin_generate_from_comment]{addin_generate_from_comment()}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/addins.R
\name{addin_settings}
\alias{addin_settings}
\title{Open the LLMcoder settings dialog}
\usage{
addin_settings()
}
\value{
Invisible \code{NULL} (called for side-effects).
}
\description{
Launches an interactive Shiny gadget that lets you configure the LLM
provider, model, API key, Ollama URL (for local models), custom base URL
(for LM Studio / vLLM / llama.cpp), and context-window size.  Settings can
optionally be persisted to \verb{~/.Rprofile} so they survive R restarts.
}
\seealso{
\code{\link[=llmcoder_setup]{llmcoder_setup()}}, \code{\link[=llmcoder_config]{llmcoder_config()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api.R
\name{build_system_prompt}
\alias{build_system_prompt}
\title{System prompt for code generation}
\usage{
build_system_prompt()
}
\description{
System prompt for code generation
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api.R
\name{call_openai_compat}
\alias{call_openai_compat}
\title{Generic OpenAI-compatible chat completions call}
\usage{
call_openai_compat(
  prompt,
  system_prompt,
  api_key,
  model,
  base_url,
  extra_hdrs = character()
)
}
\description{
Generic OpenAI-compatible chat completions call
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/editor.R
\name{get_editor_ctx}
\alias{get_editor_ctx}
\title{Get the active source editor context}
\usage{
get_editor_ctx()
}
\value{
An \code{rstudio_editor_context} object (a list returned by
\code{\link[rstudioapi:rstudio-editors]{rstudioapi::getSourceEditorContext()}}).
}
\description{
Wraps \code{\link[rstudioapi:rstudio-editors]{rstudioapi::getSourceEditorContext()}} with a check that RStudio is
available.  Called by all addin entry points before any other operation.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chat.R
\name{.js_esc}
\alias{.js_esc}
\title{Escape a character string for safe embedding in a JS string literal}
\usage{
.js_esc(x)
}
\arguments{
\item{x}{Character vector.}
}
\value{
Character vector with escapes applied.
}
\description{
Replaces characters that break JS string literals: backslash, double-quote,
newline, carriage-return, and tab.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api.R
\name{call_anthropic}
\alias{call_anthropic}
\title{Anthropic Messages API call}
\usage{
call_anthropic(prompt, system_prompt, api_key, model)
}
\description{
Anthropic Messages API call
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/session-context.R
\name{session_context_report}
\alias{session_context_report}
\title{Capture a human-readable report of the current R session state}
\usage{
session_context_report(max_objs = 20L, max_hist = 30L, quiet = FALSE)
}
\arguments{
\item{max_objs}{Maximum number of global objects to list per class group
(default 20).}

\item{max_hist}{Maximum number of console history lines to include
(default 30).}

\item{quiet}{If \code{TRUE}, suppress the \code{message()} emitted when
RStudio API is unavailable (default \code{FALSE}).}
}
\value{
Character string. A multi-section report ready to embed in a
system prompt.
}
\description{
\code{session_context_report()} collects and formats the following information
from the current R session:
\itemize{
\item R version and operating system.
\item Loaded add-on packages (non-base).
\item Global environment objects grouped by class.
\item Contents of the active source editor (via \pkg{rstudioapi}).
\item Console command history (via \pkg{rstudioapi}; falls back to
\verb{~/.Rhistory}).
}

This report is primarily used internally to populate the system prompt
sent to the LLM in the \code{\link[=addin_chat_panel]{addin_chat_panel()}} gadget, so the model has
full awareness of the analyst's working environment.
}
\examples{
\dontrun{
report <- session_context_report()
cat(report)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api.R
\name{call_openai_compat_history}
\alias{call_openai_compat_history}
\title{OpenAI-compatible multi-turn call}
\usage{
call_openai_compat_history(
  messages,
  api_key,
  model,
  base_url,
  extra_hdrs = character()
)
}
\arguments{
\item{messages}{List of message objects.}

\item{api_key}{API key.}

\item{model}{Model name.}

\item{base_url}{API base URL.}

\item{extra_hdrs}{Named character vector of extra headers.}
}
\description{
OpenAI-compatible multi-turn call
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chat.R
\name{safe_call_llm_history}
\alias{safe_call_llm_history}
\title{Safely call the LLM with full message history}
\usage{
safe_call_llm_history(messages, system_prompt_override = NULL)
}
\arguments{
\item{messages}{List of messages, each \code{list(role = ..., content = ...)}.
The last message must have \code{role = "user"}.}

\item{system_prompt_override}{Override system prompt (optional).}
}
\value{
Either a character string (response text) or
\code{list(error = message)} on failure.
}
\description{
Safely call the LLM with full message history
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/session-context.R
\name{session_context_prompt}
\alias{session_context_prompt}
\title{Build a session-context system-prompt block}
\usage{
session_context_prompt(...)
}
\arguments{
\item{...}{Passed to \code{\link[=session_context_report]{session_context_report()}}.}
}
\value{
Character string suitable for prepending to a system prompt.
}
\description{
Convenience wrapper around \code{\link[=session_context_report]{session_context_report()}} that wraps the report
in a descriptive header so the LLM can distinguish it from user content.
}
\examples{
\dontrun{
ctx_prompt <- session_context_prompt()
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api.R
\name{call_anthropic_history}
\alias{call_anthropic_history}
\title{Anthropic Messages API with multi-turn history}
\usage{
call_anthropic_history(messages, api_key, model)
}
\arguments{
\item{messages}{List of message objects.}

\item{api_key}{API key.}

\item{model}{Model name.}
}
\description{
Extracts the system role from \code{messages} and moves it to the top-level
\code{system} field, as required by the Anthropic API.  Only
\code{user} and \code{assistant} roles are allowed.
}
\keyword{internal}
